/* browseform.c
Copyright (C) 1999-2000 Ben Darnell
This file is a part of ThoughtStream,
which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static void BrowseListDrawItem(Word itemnum, RectanglePtr bounds,
			       WordPtr indices)
{
   VoidHand hrec;
   Ptr prec, text;
   
   CALLBACK_PROLOGUE;
   hrec=DmQueryRecord(IdeaDB, indices[itemnum]);
   prec=MemHandleLock(hrec);
   text=prec+IdeaNameOffset;
   WinDrawChars(text, StrLen(text), bounds->topLeft.x, bounds->topLeft.y);
   MemHandleUnlock(hrec);
   CALLBACK_EPILOGUE;
}

static Int BrowseCompare(WordPtr a, WordPtr b, Long other)
{
   Int result;
   VoidHand ha, hb;
   Ptr pa, pb;

   CALLBACK_PROLOGUE;
   ha=DmQueryRecord(IdeaDB, *a);
   pa=MemHandleLock(ha);
   hb=DmQueryRecord(IdeaDB, *b);
   pb=MemHandleLock(hb);
   result=StrCaselessCompare(pa+IdeaNameOffset, pb+IdeaNameOffset);
   MemHandleUnlock(ha);
   MemHandleUnlock(hb);
   CALLBACK_EPILOGUE;
   return result;
}

static void BrowseFormInit()
{
   FormPtr frm;
   ListPtr list;
   Word* indices;
   int count, i;

   frm=FrmGetActiveForm();
   FrmSetFocus(frm, FrmGetObjectIndex(frm, BrowseLookupField));
   list=GetObjectPtr(BrowseList);

   count=DmNumRecords(IdeaDB);
   indices=MemPtrNew(count*sizeof(Word));
   for (i=0; i<count; i++)
      indices[i]=i;
   SysQSort(indices, count, sizeof(Word), (CmpFuncPtr)BrowseCompare, 0);
   LstSetListChoices(list, (char**)indices, count);
   LstSetDrawFunction(list, (ListDrawDataFuncPtr)BrowseListDrawItem);
   LstDrawList(list);
}

Boolean BrowseFormHandleEvent(EventPtr e)
{
   Boolean handled=false;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      FrmDrawForm(FrmGetActiveForm());
      BrowseFormInit();
      handled=true;
      break;

   case ctlSelectEvent: {
      EventType close;
      Word recindex=-1;
      ListPtr list;
      
      switch (e->data.ctlSelect.controlID) {
      case OK: BrowseSelectItem:
	 list=GetObjectPtr(BrowseList);
	 if (LstGetSelection(list)>=0)
	    recindex=((WordPtr)list->itemsText)[LstGetSelection(list)];
	 /* fall thru */
	 
      case Cancel: {
	 list=GetObjectPtr(BrowseList);
	 MemSet(&close, sizeof(EventType), 0);
	 close.eType=dlgCloseEvent;
	 close.data.generic.data1=recindex;
	 EvtAddEventToQueue(&close);
	 MemPtrFree(list->itemsText);
	 FrmReturnToForm(0);
	 handled=true;
	 break;
      }
	 
      };
      break;
   } /* end of case ctlSelectEvent */

   case lstSelectEvent:
      goto BrowseSelectItem;
      break;

   case keyDownEvent: {
      ListPtr list=GetObjectPtr(BrowseList);
      FieldPtr fld=GetObjectPtr(BrowseLookupField);
      VoidHand htext;
      CharPtr ptext;
      WordPtr indices;
      int i;
      
      if (e->data.keyDown.chr==pageUpChr) {
	 LstScrollList(list, up, LstGetVisibleItems(list)-1);
	 break;
      } else if (e->data.keyDown.chr==pageDownChr) {
	 LstScrollList(list, down, LstGetVisibleItems(list)-1);
	 break;
      } else if (e->data.keyDown.chr=='\n') {
	 goto BrowseSelectItem;
      };
      
      fld=GetObjectPtr(BrowseLookupField);
      if (!FldHandleEvent(fld, e))
	 break;
      htext=(VoidHand)FldGetTextHandle(fld);
      ptext=MemHandleLock(htext);
      indices=(WordPtr)list->itemsText;
      for (i=0; i<DmNumRecords(IdeaDB); i++) {
	 Boolean match=true;
	 VoidHand hrec;
	 Ptr prec;

	 hrec=DmQueryRecord(IdeaDB, indices[i]);
	 if (!hrec)
	    match=false;
	 else {
	    prec=MemHandleLock(hrec);
	    if (StrNCaselessCompare(prec+IdeaNameOffset, ptext, StrLen(ptext)))
	       match=false;
	    MemHandleUnlock(hrec);
	    if (match) {
	       LstSetSelection(list, i);
	       break;
	    };
	 };
      };
      MemHandleUnlock(htext);
      handled=true;
      break;
   }
	    

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}

Word BrowseForRecord()
{
   FrmPopupForm(BrowseForm);
   return EventLoop();
}
