/* fieldform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static union {
   ULong time;
} FieldData;

static void FieldTimeUpdate()
{
   DateTimeType datetime;
   ControlPtr ctl;
   Char datebuf[dateStringLength], timebuf[timeStringLength];   
   
   TimSecondsToDateTime(FieldData.time, &datetime);
   DateToAscii(datetime.month, datetime.day, datetime.year,
	       (DateFormatType)PrefGetPreference(prefDateFormat), datebuf);
   TimeToAscii(datetime.hour, datetime.minute,
	       (TimeFormatType)PrefGetPreference(prefTimeFormat), timebuf);
   
   ctl=GetObjectPtr(FieldTimeDateSelector);
   CtlSetLabel(ctl, datebuf);
   CtlShowControl(ctl);
   
   ctl=GetObjectPtr(FieldTimeTimeSelector);
   CtlSetLabel(ctl, timebuf);
   CtlShowControl(ctl);
};


static void FieldSetMode(Word mode, Boolean clear)
{
   FormPtr frm;
   FieldPtr fld;
   VoidHand hrec;
   Ptr prec;
   Word itemoffset;
   CharPtr pdata;
   ControlPtr ctl;

   frm=FrmGetActiveForm();
   
   FrmHideObject(frm, FrmGetObjectIndex(frm, FieldTextField));
   CtlHideControl(GetObjectPtr(FieldCheckBox));
   CtlHideControl(GetObjectPtr(FieldTimeDateSelector));
   CtlHideControl(GetObjectPtr(FieldTimeTimeSelector));
   CtlHideControl(GetObjectPtr(FieldMultiTextEditButton));
   FrmHideObject(frm, FrmGetObjectIndex(frm, FieldNumberField));
   CtlHideControl(GetObjectPtr(FieldNumberPlusButton));
   CtlHideControl(GetObjectPtr(FieldNumberMinusButton));
   FrmHideObject(frm, FrmGetObjectIndex(frm, FieldIncrementField));

   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);
   pdata=prec+itemoffset+FieldDataOffset(prec+itemoffset);
   
   switch (mode) {
   case FieldTypeText:
      fld=GetObjectPtr(FieldTextField);
      if (clear) {
	 FldDelete(fld, 0, FldGetTextLength(fld));
	 FldSetDirty(fld, true);
      } else {
	 FldInsert(fld, pdata, StrLen(pdata));
	 FldSetDirty(fld, false);
      }
      FrmShowObject(frm, FrmGetObjectIndex(frm, FieldTextField));
      break;

   case FieldTypeBoolean:
      ctl=GetObjectPtr(FieldCheckBox);
      if (clear)
	 CtlSetValue(ctl, false);
      else
	 CtlSetValue(ctl, (pdata[0]=='y'));
      CtlShowControl(ctl);
      break;

   case FieldTypeTime: {
      if (clear) 
	 FieldData.time=TimGetSeconds();
      else
	 FieldData.time=StrAToI(pdata);
      FieldTimeUpdate();
      break;
   };

   case FieldTypeMultiText:
      /* note: this currently ignores the clear param */
      ctl=GetObjectPtr(FieldMultiTextEditButton);
      CtlShowControl(ctl);
      break;

   case FieldTypeNumber:
      ctl=GetObjectPtr(FieldNumberPlusButton);
      CtlShowControl(ctl);
      ctl=GetObjectPtr(FieldNumberMinusButton);
      CtlShowControl(ctl);
      
      fld=GetObjectPtr(FieldNumberField);
      FldDelete(fld, 0, FldGetTextLength(fld));
      if (clear)
	 FldInsert(fld, "0", 1);
      else
	 FldInsert(fld, pdata, StrLen(pdata));
      FldSetDirty(fld, false);
      FrmShowObject(frm, FrmGetObjectIndex(frm, FieldNumberField));

      fld=GetObjectPtr(FieldIncrementField);
      FldDelete(fld, 0, FldGetTextLength(fld));
      FldInsert(fld, "1", 1);
      FrmShowObject(frm, FrmGetObjectIndex(frm, FieldIncrementField));
      break;

   default:
      break;
   }
   MemHandleUnlock(hrec);
}

static void FieldFormInit()
{
   FieldPtr namefield;
   VoidHand hrec;
   Ptr prec;
   Word itemoffset;
   Char mode;

   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);

   namefield=GetObjectPtr(FieldNameField);
   FldInsert(namefield, prec+itemoffset+FieldNameOffset,
	     StrLen(prec+itemoffset+FieldNameOffset));
   FldSetDirty(namefield, false);

   mode=GetByte(prec, itemoffset+FieldTypeOffset);
   FieldSetMode(mode, false);
   switch (mode) {
   case FieldTypeText:
      CtlSetValue(GetObjectPtr(FieldTextPushButton), true);
      break;

   case FieldTypeBoolean:
      CtlSetValue(GetObjectPtr(FieldCheckBoxPushButton), true);
      break;

   case FieldTypeTime:
      CtlSetValue(GetObjectPtr(FieldTimePushButton), true);
      break;

   case FieldTypeMultiText:
      CtlSetValue(GetObjectPtr(FieldMultiTextPushButton), true);
      break;

   case FieldTypeNumber:
      CtlSetValue(GetObjectPtr(FieldNumberPushButton), true);
      break;
   }

   MemHandleUnlock(hrec);
}

static void FieldFormCleanup(Boolean save)
{
   VoidHand hrec;
   Ptr prec;
   Word itemoffset;
   Char fieldtype;
   Boolean dirty=false;
   FieldPtr fld;
   Word dataoffset, datalen;

   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);

   fld=GetObjectPtr(FieldNameField);
   if (FldDirty(fld)) {
      Word namelen=StrLen(prec+itemoffset+FieldNameOffset);
      CharPtr newname;
      
      MemHandleUnlock(hrec);
      DBRemove(IdeaDB, ActiveItemIdeaIndex, itemoffset+FieldNameOffset,
	       namelen);
      newname=FldGetTextPtr(fld);
      if (newname&&newname[0])
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, itemoffset+FieldNameOffset,
		  newname, StrLen(newname));
      hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
      prec=MemHandleLock(hrec);
   };

   MemHandleUnlock(hrec);
   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   
   dataoffset=FieldDataOffset(prec+itemoffset);
   datalen=StrLen(prec+itemoffset+dataoffset);
   
   fieldtype=GetByte(prec, itemoffset+FieldTypeOffset);
   
   if (save) {
      if (CtlGetValue(GetObjectPtr(FieldTextPushButton))) {
	 CharPtr newdata;
	 
	 if (fieldtype!=FieldTypeText) {
	    SetByte(prec, itemoffset+FieldTypeOffset, FieldTypeText);
	    dirty=true;
	 };
	 MemHandleUnlock(hrec);
	 fld=GetObjectPtr(FieldTextField);
	 if (!dirty) dirty=FldDirty(fld);

	 if (dirty)
	    DBRemove(IdeaDB, ActiveItemIdeaIndex, itemoffset+dataoffset,
		     datalen);

	 newdata=FldGetTextPtr(fld);
	 if (newdata&&newdata[0])
	    DBInsert(IdeaDB, ActiveItemIdeaIndex, itemoffset+dataoffset,
		     newdata, StrLen(newdata));
	 
      } else if (CtlGetValue(GetObjectPtr(FieldCheckBoxPushButton))) {
	 Boolean newdata, olddata;
	 if (fieldtype!=FieldTypeBoolean) {
	    SetByte(prec, itemoffset+FieldTypeOffset, FieldTypeBoolean);
	    dirty=true;
	 };

	 newdata=CtlGetValue(GetObjectPtr(FieldCheckBox));
	 olddata=((prec+itemoffset+dataoffset)[0]=='y');
	 if (dirty||(newdata!=olddata)) {
	    if (newdata)
	       SetByte(prec, itemoffset+dataoffset, 'y');
	    else
	       SetByte(prec, itemoffset+dataoffset, 'n');
	 };
	 MemHandleUnlock(hrec);
	 
      } else if (CtlGetValue(GetObjectPtr(FieldTimePushButton))) {
	 CharPtr newdata;
	 
	 if (fieldtype!=FieldTypeTime) {
	    SetByte(prec, itemoffset+FieldTypeOffset, FieldTypeTime);
	    dirty=true;
	 };
	 MemHandleUnlock(hrec);
	 
	 DBRemove(IdeaDB, ActiveItemIdeaIndex, itemoffset+dataoffset,
		  datalen);

	 newdata=MemPtrNew(12);
	 StrIToA(newdata, FieldData.time);
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, itemoffset+dataoffset,
		  newdata, StrLen(newdata));
	 MemPtrFree(newdata);
	 
      } else if (CtlGetValue(GetObjectPtr(FieldMultiTextPushButton))) {
	 /* FIXME: FieldFormCleanup multitext */
	 MemHandleUnlock(hrec);
      } else if (CtlGetValue(GetObjectPtr(FieldNumberPushButton))) {
	 /* FIXME: FieldFormCleanup number */
	 MemHandleUnlock(hrec);
      } else
	 MemHandleUnlock(hrec);

      prec=MemHandleLock(hrec);
      SetWord(prec, itemoffset+ItemSizeOffset,
	      1+FieldDataOffset(prec+itemoffset)
	      +StrLen(prec+itemoffset+FieldDataOffset(prec+itemoffset)));
      MemHandleUnlock(hrec);
   }
   
   if (dirty)
      IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);
   DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, dirty);

   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

Boolean FieldFormHandleEvent(EventPtr e)
{
   Boolean handled=false;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      FrmDrawForm(FrmGetActiveForm());
      FieldFormInit();
      handled=true;
      break;

   case frmCloseEvent:
      FieldFormCleanup(false);
      break;

   case ctlSelectEvent:
      switch (e->data.ctlSelect.controlID) {
      case DoneButton:
	 FieldFormCleanup(true);
	 FrmReturnToForm(0);
	 handled=true;
	 break;

      case CancelButton:
	 FieldFormCleanup(false);
	 FrmReturnToForm(0);
	 handled=true;
	 break;

      case FieldTextPushButton:
	 FieldSetMode(FieldTypeText, true);
	 break;

      case FieldCheckBoxPushButton:
	 FieldSetMode(FieldTypeBoolean, true);
	 break;

      case FieldTimePushButton:
	 FieldSetMode(FieldTypeTime, true);
	 break;

      case FieldMultiTextPushButton:
	 FieldSetMode(FieldTypeMultiText, true);
	 break;

      case FieldNumberPushButton:
	 FieldSetMode(FieldTypeNumber, true);
	 break;

      case FieldTimeDateSelector: {
	 DateTimeType datetime;
	 TimSecondsToDateTime(FieldData.time, &datetime);
	 if (SelectDay(selectDayByDay, &datetime.month,
		       &datetime.day, &datetime.year, "Select Day")) {
	    FieldData.time=TimDateTimeToSeconds(&datetime);
	    FieldTimeUpdate();
	 };
	 break;
      };

      case FieldTimeTimeSelector: {
	 DateTimeType datetime;
	 ULong romversion=0;
	 
	 TimSecondsToDateTime(FieldData.time, &datetime);
	 if (0&&(!FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion))
	     &&(romversion>0x03103000)) {
	    /* we're running on 3.1 or higher, so we have SelectOneTime */
	    /* FIXME: need 3.1 sdk for SelectOneTime */
	 } else {
	    /* use SelectTime on old OS; disregard ending time */
	    TimeType starttime, endtime;
	    starttime.hours=endtime.hours=datetime.hour;
	    starttime.minutes=endtime.minutes=datetime.minute;
	    if (SelectTime(&starttime, &endtime, false, "Select Time", 8)) {
	       datetime.hour=starttime.hours;
	       datetime.minute=starttime.minutes;
	       FieldData.time=TimDateTimeToSeconds(&datetime);
	       FieldTimeUpdate();
	    };
	 };
	 break;
      };

      case FieldMultiTextEditButton:
	 /* FIXME: FieldMultiTextEditButton */
	 break;

      default:
	 break;
      };
      break;

   case menuEvent:
      handled=EditMenuHandleEvent(e->data.menu.itemID);
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
