/* idea.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include <CharAttr.h>
#include "callback.h"

#include "thoughtstream.h"

ULong IdeaGenID()
{
   /* this should probably be made more definitely unique */
   return Random32();
}

Word IdeaCalcHeaderSize(CharPtr name)
{
   return IdeaNameOffset+StrLen(name)+1;
}

Word IdeaNew(DmOpenRef db, CharPtr name)
{
   VoidHand hrec;
   Ptr prec;
   int hdr_size;
   UInt index=dmMaxRecordIndex;
   Word item_size=4;

   hdr_size=IdeaCalcHeaderSize(name);
   hrec=DmNewRecord(db, &index, hdr_size+item_size);
   prec=MemHandleLock(hrec);
   SetLong(prec, IdeaIDOffset, IdeaGenID());
   SetWord(prec, IdeaHeaderSizeOffset, hdr_size);
   SetByte(prec, IdeaScoreOffset, 128);
   DmWrite(prec, IdeaNameOffset, name, StrLen(name)+1);

   SetWord(prec, ItemOffset(prec, 0)+ItemSizeOffset, item_size);
   SetByte(prec, ItemOffset(prec, 0)+ItemTypeOffset, NoteType);
   DmWrite(prec, ItemOffset(prec, 0)+NoteTextOffset, "", 1);

   MemHandleUnlock(hrec);
   DmReleaseRecord(db, index, true);
   IdeaSetDirty(db, index);

   return index;
}

Word ItemOffset(Ptr pidea, Word index)
{
   Word i, result;

   result=GetWord(pidea, IdeaHeaderSizeOffset);
   for (i=0; i<index; i++)
      result+=GetWord(pidea, result);
   return result;
}

Word IdeaNumItems(Ptr pidea)
{
   Word pos=GetWord(pidea, IdeaHeaderSizeOffset);
   Word max=MemPtrSize(pidea);
   Word count=0;
   
   while (pos<max) {
      pos+=GetWord(pidea, pos);
      count++;
   };
   return count;
}

Word IdeaLookup(DmOpenRef db, ULong id)
{
   int i;
   VoidHand hrec;
   Ptr prec;
   ULong tmp;
   
   for (i=0; i<DmNumRecords(db); i++) {
      hrec=DmQueryRecord(db, i);
      if (MemHandleSize(hrec)==4)
	 continue; /* the idea has been deleted */
      prec=MemHandleLock(hrec);
      tmp=GetLong(prec, IdeaIDOffset);
      MemHandleUnlock(hrec);
      if (tmp==id)
	 return i;
   };
   return dmMaxRecordIndex;
}

/* based on the system find dialog */
static void SearchActivate()
{
   VoidHand hrec, hfindparams;
   Ptr prec;
   FindParamsPtr pfindparams;
   Word itemoffset;
   SystemPreferencesType sysprefs;
   BytePtr convtbl;
   int len, i;
   DmSearchStateType dmsearchstate;
   FormPtr findresults;
   SWord selection=0;
   TablePtr tbl;
   Word row;
   GoToParamsPtr pgoto;
   DWord dummy;

   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);

   if (!(*(prec+itemoffset+SearchTextOffset))) {
      /* no search text, so just launch app */
      UInt cardno;
      LocalID dbid;
      ULong creator;
      
      creator=GetLong(prec, itemoffset+SearchCreatorOffset);
      if (!creator) {
	 FrmPopupForm(SearchForm);
	 return;
      };
      DmGetNextDatabaseByTypeCreator(true, &dmsearchstate,
				     'appl',
				     creator,
				     true, &cardno, &dbid);
      SysUIAppSwitch(cardno, dbid, sysAppLaunchCmdNormalLaunch, 0);
      MemHandleUnlock(hrec);
      return;
   }

   /* initialize find params */
   hfindparams=MemHandleNew(sizeof(FindParamsType));
   pfindparams=(FindParamsPtr)MemHandleLock(hfindparams);
   MemSet(pfindparams, sizeof(FindParamsType), 0);
   pfindparams->newSearch=true;
   pfindparams->lineNumber=0;

   len=StrLen(prec+itemoffset+SearchTextOffset);
   if (len>maxFindStrLen)
      len=maxFindStrLen;
   StrNCopy(pfindparams->strAsTyped, prec+itemoffset+SearchTextOffset, len+1);
   pfindparams->strAsTyped[maxFindStrLen]=0;
   convtbl=GetCharCaselessValue();
   for (i=0; i<len; i++)
      pfindparams->strToFind[i]=convtbl[pfindparams->strAsTyped[i]];
   pfindparams->strToFind[len]=0;

   PrefGetPreferences(&sysprefs);
   if (sysprefs.hideSecretRecords)
      pfindparams->dbAccesMode=dmModeReadOnly;
   else
      pfindparams->dbAccesMode=dmModeReadOnly|dmModeShowSecret;

   if (SysCurAppDatabase(&pfindparams->callerAppCardNo,
			 &pfindparams->callerAppDbID))
      ErrDisplay("Can't happen");

   DmGetNextDatabaseByTypeCreator(true, &dmsearchstate, 'appl',
				  GetLong(prec,
					  itemoffset+SearchCreatorOffset),
				  true,
				  &pfindparams->appCardNo,
				  &pfindparams->appDbID);

   MemHandleUnlock(hrec);

   /* do the search */
   findresults=FrmInitForm(FindResultsDialog);
   FrmSetActiveForm(findresults);
   tbl=GetObjectPtr(FindResultsTable);
   TblSetColumnUsable(tbl, 0, true);
   for (row=0; row<TblGetNumberOfRows(tbl); row++) {
      TblSetRowUsable(tbl, row, false);
      TblSetRowSelectable(tbl, row, false);
   };
   FrmDrawForm(findresults);
   CtlHideControl(GetObjectPtr(FindResultsStopButton));
   CtlShowControl(GetObjectPtr(FindResultsCancelButton));
   while(EvtEventAvail()) {
      /* must flush event queue before starting find */
      EventType e;
      EvtGetEvent(&e, 0);
      SysHandleEvent(&e);
   };
   if (SysAppLaunch(pfindparams->appCardNo, pfindparams->appDbID, 0,
		    sysAppLaunchCmdFind, (Ptr)pfindparams, &dummy)) {
      FrmAlert(SearchErrorAlert);
      goto exit;
   };

   /* find the selection */
   if (pfindparams->numMatches<1) {
      FrmAlert(SearchNotFoundAlert);
      goto exit;
   };
   if (pfindparams->numMatches==1)
      selection=0;
   else {
      Boolean done=false;
      EventType event;

      while (!done) {
	 EvtGetEvent(&event, 0);
	 FrmHandleEvent(findresults, &event);
	 if (SysHandleEvent(&event))
	    continue;
	 switch (event.eType) {
	 case tblSelectEvent:
	    selection=event.data.tblSelect.row-1;
	    done=true;
	    break;

	 case ctlSelectEvent:
	    if (event.data.ctlSelect.controlID==FindResultsCancelButton) {
	       selection=-1;
	       done=true;
	    };
	    break;

	 case appStopEvent:
	    EvtAddEventToQueue(&event);
	    break;

	 default: break;
	 };
      };
   };

   if (selection==-1)
      goto exit;

   /* go to the selection */
   pgoto=MemPtrNew(sizeof(GoToParamsType));
   MemPtrSetOwner(pgoto, 0);
   pgoto->searchStrLen=len;
   pgoto->dbCardNo=pfindparams->match[selection].dbCardNo;
   pgoto->dbID=pfindparams->match[selection].dbID;
   pgoto->recordNum=pfindparams->match[selection].recordNum;
   pgoto->matchPos=pfindparams->match[selection].matchPos;
   pgoto->matchFieldNum=pfindparams->match[selection].matchFieldNum;
   pgoto->matchCustom=pfindparams->match[selection].matchCustom;
   SysUIAppSwitch(pfindparams->match[selection].appCardNo,
		  pfindparams->match[selection].appDbID,
		  sysAppLaunchCmdGoTo,
		  (Ptr)pgoto);

   FrmDeleteForm(findresults);
   MemHandleUnlock(hfindparams);
   MemHandleFree(hfindparams);
   return;

 exit:
   FrmEraseForm(findresults);
   FrmDeleteForm(findresults);
   FrmSetActiveForm(FrmGetFormPtr(MainForm));
   MemHandleUnlock(hfindparams);
   MemHandleFree(hfindparams);
   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

void ItemActivate(Word ideaindex, SWord itemindex, Word mode)
{
   VoidHand hrec;
   Ptr prec;
   Byte type;
   Word itemoff;

   if (itemindex==-1) {
      /* title */
      /* FIXME: activate on title */
      return;
   };
   
   ActiveItemIdeaIndex=ideaindex;
   ActiveItemIndex=itemindex;
   
   hrec=DmQueryRecord(IdeaDB, ideaindex);
   prec=MemHandleLock(hrec);
   itemoff=ItemOffset(prec, itemindex);
   type=GetByte(prec, itemoff+ItemTypeOffset);
      
   switch (type) {
   case NoteType:
      FrmPopupForm(NoteForm);
      break;

   case ParentType:
   case ChildType:
   case PeerType:
      switch (mode) {
      case ActivateDefault: {
	 ULong target_id;

	 target_id=GetLong(prec, itemoff+LinkTargetIDOffset);
	 if (!target_id)
	    FrmPopupForm(LinkForm);
	 else {
	    CharPtr mindref=prec+itemoff+LinkTargetMindOffset(prec+itemoff);

	    if (mindref[0]) {
	       /* FIXME
		  int i;
	       VoidHand href=0;
	       Ptr pref=0, p;
	       CharPtr name=0;

	       for (i=0; i<DmNumRecords(MindrefDB); i++) {
		  href=DmQueryRecord(MindrefDB, i);
		  pref=MemHandleLock(href);
		  if (!StrCompare(pref, mindref))
		     break;
		  MemHandleLock(href);
		  pref=NULL;
	       }
	       if (!pref) break;
	       p=pref+StrLen(pref)+2;
	       while (p-pref<MemHandleSize(href)) {
		  if (p[0]==PalmDBMindref) {
		     name=p+1;
		     break;
		  };
	       }
	       if (!name) break;
	       MindClose();
	       MindOpen(name);
	       MemHandleUnlock(href);
	       */
	    }
	    CurrentID=target_id;
	    WinEraseWindow();
	    MainFormInit(FrmGetActiveForm());
	    FrmDrawForm(FrmGetActiveForm());
	    MainFormDraw();
	 };
	 break;
      };
      
      case ActivateEdit:
	 FrmPopupForm(LinkForm);
	 break;
      };
      break;

   case URLType:
      FrmPopupForm(URLForm);
      break;

   case SearchType:
      switch (mode) {
      case ActivateDefault: 
	 SearchActivate();
	 break;

      case ActivateEdit:
	 FrmPopupForm(SearchForm);
	 break;
      };
      break;

   case LMType:
      switch (mode) {
      case ActivateDefault:
	 LMFollowLink();
	 break;

      case ActivateEdit:
	 FrmPopupForm(LMForm);
	 break;
      };
      break;

   case FieldType:
      switch (mode) {
      case ActivateDefault:
	 /* FIXME ItemActivate FieldType ActivateDefault */
	 break;

      case ActivateEdit:
	 FrmPopupForm(FieldForm);
	 break;
      };
      break;
      
   };
   MemHandleUnlock(hrec);
}

void ItemDelete(DmOpenRef db, Word ideaindex, Word itemindex)
{
   VoidHand hrec;
   Ptr prec;
   Word itemoffset, itemlen;
   Byte type;

   hrec=DmQueryRecord(db, ideaindex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, itemindex);
   
   /* if deleting link, should also delete inverse */   
   type=GetByte(prec, itemoffset+ItemTypeOffset);
   if ((type==ParentType)||(type==ChildType)||(type==PeerType)) {
      Word targetindex;
      
      targetindex=IdeaLookup(IdeaDB,
			     GetLong(prec, itemoffset+LinkTargetIDOffset));
      if ((GetByte(prec, itemoffset+LinkTargetMindOffset(prec+itemoffset))==0)
	  &&(targetindex!=-1)) {
	 VoidHand htarget;
	 Ptr ptarget;
	 int i;
	 Word inv_offset=0, inv_len;
	 Byte inv_dir;

	 switch (GetByte(prec, itemoffset+ItemTypeOffset)) {
	 case ParentType: inv_dir=ChildType;  break;
	 case ChildType:  inv_dir=ParentType; break;
	 case PeerType:   inv_dir=PeerType;   break;
	 };
	 
	 htarget=DmQueryRecord(IdeaDB, targetindex);
	 ptarget=MemHandleLock(htarget);
	 for (i=0; i<IdeaNumItems(ptarget); i++) {
	    Word offset=ItemOffset(ptarget, i);

	    if (GetByte(ptarget, offset+ItemTypeOffset)!=inv_dir)
	       continue;
	    if (GetByte(ptarget,
			offset+LinkTargetMindOffset(prec+itemoffset)))
	       continue;
	    if (GetLong(ptarget, offset+LinkTargetIDOffset)==CurrentID) {
	       inv_offset=offset;
	       break;
	    };
	 };
	 if (inv_offset) {
	    inv_len=GetWord(ptarget, inv_offset+ItemSizeOffset);
	    MemHandleUnlock(htarget);
	    DBRemove(db, targetindex, inv_offset, inv_len);
	 } else
	    MemHandleUnlock(htarget);
      };
   };
	       
   itemlen=GetWord(prec, itemoffset+ItemSizeOffset);
   MemHandleUnlock(hrec);
   DBRemove(db, ideaindex, itemoffset, itemlen);
}

void IdeaDelete(DmOpenRef db, Word ideaindex)
{
   int i;
   VoidHand hrec;
   Ptr prec;
   UInt attr;

   hrec=DmQueryRecord(db, ideaindex);
   prec=MemHandleLock(hrec);
   i=IdeaNumItems(prec);
   MemHandleUnlock(hrec);
   
   for (; i>=0; i--) {
      ItemDelete(db, ideaindex, i);
   }

   /* keep the idea id around for syncing */
   DmResizeRecord(db, ideaindex, 4);
   IdeaSetDirty(db, ideaindex);

   /* if we don't have a syncing partner, then really delete the idea */
   DmRecordInfo(db, ideaindex, &attr, NULL, NULL);
   if (!(attr&dmRecAttrCategoryMask))
      DmRemoveRecord(db, ideaindex);
};



Word ItemNew(DmOpenRef db, Word idea_index, Byte item_type)
{
   Char note_data[]={0,4, NoteType, 0};
   Char link_data[]={0,11, ChildType, 0,0,0,0, 0, 0,0,0};
   Char url_data[]={0,5, URLType, 0, 0};
   Char search_data[]={0,8, SearchType, 0,0,0,0, 0};
   Char field_data[]={0, 6, FieldType, FieldTypeText, 0, 0};
   VoidHand hrec;
   Ptr prec;
   Word item_index;

   switch (item_type) {
   case NoteType:
      DBAppend(db, idea_index, note_data, note_data[1]);
      break;

   case ChildType:
      DBAppend(db, idea_index, link_data, link_data[1]);
      break;

   case URLType:
      DBAppend(db, idea_index, url_data, url_data[1]);
      break;

   case SearchType:
      DBAppend(db, idea_index, search_data, search_data[1]);
      break;

   case FieldType:
      DBAppend(db, idea_index, field_data, field_data[1]);
      break;
   };

   hrec=DmGetRecord(db, idea_index);
   prec=MemHandleLock(hrec);
   item_index=IdeaNumItems(prec)-1;
   IdeaSetDirty(IdeaDB, idea_index);
   MemHandleUnlock(hrec);
   DmReleaseRecord(db, idea_index, true);
   return item_index;
}

SWord IdeaFindByName(DmOpenRef db, CharPtr name)
{
   Word i, numrecords;

   numrecords=DmNumRecords(db);
   for (i=0; i<numrecords; i++) {
      VoidHand hrec;
      Ptr prec;

      hrec=DmQueryRecord(db, i);
      if (MemHandleSize(hrec)==4)
	 continue; /* the idea has been deleted */
      prec=MemHandleLock(hrec);
      if (!StrCaselessCompare(prec+IdeaNameOffset, name)) {
	 MemHandleUnlock(hrec);
	 return i;
      };
      MemHandleUnlock(hrec);
   }
   return -1;
}

void LMStoreLink(TSLinkPtr linkinfo)
{
   DmOpenRef db;
   LocalID dbid;
   Word index;
   VoidHand hrec;
   Ptr prec;
   Ptr otherlinkinfo, title;
   int titlelen, oldsize;

   dbid=DmFindDatabase(0, linkinfo[0].db_name);
   db=DmOpenDatabase(0, dbid, dmModeReadWrite);
   index=IdeaLookup(db, linkinfo[0].id);
   otherlinkinfo=((Ptr)linkinfo)+64;
   title=otherlinkinfo+64;
   titlelen=StrLen(title)+1;
   hrec=DmQueryRecord(db, index);
   oldsize=MemHandleSize(hrec);
   DmResizeRecord(db, index, oldsize+LMTextOffset+titlelen);
   hrec=DmGetRecord(db, index);
   prec=MemHandleLock(hrec);
   SetWord(prec, oldsize+ItemSizeOffset, LMTextOffset+titlelen);
   SetByte(prec, oldsize+ItemTypeOffset, LMType);
   DmWrite(prec, oldsize+LMDataOffset, otherlinkinfo, 64+titlelen);
   MemHandleUnlock(hrec);
   DmReleaseRecord(db, index, true);
   DmCloseDatabase(db);
}

void IdeaSetDirty(DmOpenRef db, Word index)
{
   UInt attr;
   Byte dirtycount;
   UInt cardno;
   LocalID dbid;
   LocalID appinfoid;
   ULongPtr pappinfo;

   DmOpenDatabaseInfo(db, &dbid, NULL, NULL, &cardno, NULL);
   DmDatabaseInfo(cardno, dbid, NULL, NULL, NULL, NULL, NULL,
		  NULL, NULL, &appinfoid, NULL, NULL, NULL);
   if (appinfoid) {
      pappinfo=MemLocalIDToLockedPtr(appinfoid, cardno);
      for (dirtycount=0; ((dirtycount<=dmRecAttrCategoryMask)
			  &&(pappinfo[2+dirtycount])); dirtycount++) { };
      MemPtrUnlock(pappinfo);
      
      DmRecordInfo(db, index, &attr, NULL, NULL);
      attr&=~dmRecAttrCategoryMask;
      attr|=dirtycount;
      DmSetRecordInfo(db, index, &attr, NULL);
   };
}
