/* idea.h
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/
#ifndef idea_h
#define idea_h

/* idea header:
   uint32 ID;
   uint16 header_size;
   uint8 score;
   char name[];
*/
#define IdeaIDOffset 0
#define IdeaHeaderSizeOffset IdeaIDOffset+4
#define IdeaScoreOffset IdeaHeaderSizeOffset+2
#define IdeaNameOffset IdeaScoreOffset+1

Word IdeaCalcHeaderSize(CharPtr name);

/* item headers:
   uint16 item_size;
   uint8 item_type;
*/
#define ItemSizeOffset 0
#define ItemTypeOffset ItemSizeOffset+2
#define ItemHeaderSize ItemTypeOffset+1

/* item types
   don't change any of these; just add on at the end
 */
#define NoteType 0
#define ParentType 1
#define ChildType 2
#define PeerType 3
#define URLType 4
#define SearchType 5
#define LMType 6
#define FieldType 7
#define NumItemTypes 8

/* note data:
   char text[];
*/
#define NoteTextOffset ItemHeaderSize+0

/* link data:
   uint32 target_id;
   uint8 score;
   char label[];
   char target_mind[];
   char target_title[];
*/
#define LinkTargetIDOffset ItemHeaderSize+0
#define LinkScoreOffset LinkTargetIDOffset+4
#define LinkLabelOffset LinkScoreOffset+1
#define LinkTargetMindOffset(pitem) (LinkLabelOffset+StrLen((pitem)+LinkLabelOffset)+1)
#define LinkTargetTitleOffset(pitem) (LinkTargetMindOffset((pitem))+StrLen((pitem)+LinkTargetMindOffset((pitem)))+1)
#define LinkSize(pitem) (LinkTargetTitleOffset((pitem))+StrLen((pitem)+LinkTargetTitleOffset((pitem)))+1)

/* url data:
   char[] title;
   char[] address;
*/
#define URLTitleOffset ItemHeaderSize+0
#define URLAddressOffset(pitem) (URLTitleOffset+StrLen((pitem)+URLTitleOffset)+1)

/* search data:
   uint32 creator id
   char[] searchtext
 */
#define SearchCreatorOffset ItemHeaderSize+0
#define SearchTextOffset SearchCreatorOffset+4

/* linkmaster data
   char[64] data
*/
#define LMDataOffset ItemHeaderSize+0
#define LMTextOffset LMDataOffset+64

/* field data
   uint8 field type
   char[] name
   char[] data
*/
#define FieldTypeOffset ItemHeaderSize+0
#define FieldNameOffset FieldTypeOffset+1
#define FieldDataOffset(pitem) (FieldNameOffset+StrLen((pitem)+FieldNameOffset)+1)

/* don't change these, just add to the end */
/* text field: text stored in data buffer */
#define FieldTypeText 0
/* checkbox field: first char of data buffer is either 1 or 0*/
#define FieldTypeBoolean 1
/* date/time field: data buffer contains base 10 representation of a time,
   stored as seconds since 1969 */
#define FieldTypeTime 2
/* multi-line text field: text stored in data buffer */
#define FieldTypeMultiText 3
/* number field: number stored in base 10 in data buffer */
#define FieldTypeNumber 4

/* mindref data
   char[] name;
   uint8 dirty_count;
   { repeat as needed:
    uint8 type;
    char[] path;
   }
*/
#define PalmDBMindref 0
#define FileMindref 1
#define URLMindref 2

#endif /* idea_h */
