/* linkform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static Word LinkInvOffset;

/* be sure to unlock the pointer this function returns */
Ptr MindListGetItem(UInt index)
{
   VoidHand hstr;
   Ptr pstr;
   
   /*if (index==0) FIXME */
   hstr=DmGetResource('tSTR', ThisMindString);
   /*FIXME else if (index==1+DmNumRecords(MindrefDB)) 
     hstr=DmGetResource('tSTR', OtherMindString);
   else
   hstr=DmQueryRecord(MindrefDB, index-1);*/
   pstr=MemHandleLock(hstr);

   return pstr;
}   

void MindListDrawFunc(UInt index, RectanglePtr bounds, CharPtr* text)
{
   Ptr p;
   
   CALLBACK_PROLOGUE;

   p=MindListGetItem(index);
   WinDrawChars(p, StrLen(p), bounds->topLeft.x, bounds->topLeft.y);
   MemPtrUnlock(p);
   
   CALLBACK_EPILOGUE;
}

static void LinkFormInit()
{
   VoidHand hrec;
   Ptr prec;
   Word itemoffset;
   CharPtr label;
   FieldPtr labelfield, targetfield;
   ULong target_id;
   SWord target_index=0;
   VoidHand htarget=0;
   Ptr ptarget;
   int i;
   Byte inverse_dir=0;
   ListPtr list;
   Boolean remote;

   LinkInvOffset=0;

   CtlSetValue(GetObjectPtr(LinkUseExistingPushButton), 1);

   /* set direction buttons */
   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);
   switch (GetByte(prec, itemoffset+ItemTypeOffset)) {
   case ParentType:
      CtlSetValue(GetObjectPtr(LinkParentPushButton), 1);
      inverse_dir=ChildType;
      break;
      
   case ChildType:
      CtlSetValue(GetObjectPtr(LinkChildPushButton), 1);
      inverse_dir=ParentType;
      break;
      
   case PeerType:
      CtlSetValue(GetObjectPtr(LinkPeerPushButton), 1);
      inverse_dir=PeerType;
      break;  
   }

   /* check if remote link */
   if (GetByte(prec, itemoffset+LinkTargetMindOffset(prec+itemoffset)))
      remote=true;
   else
      remote=false;
   
   /* lookup inverse link */
   if (!remote) {
      target_id=GetLong(prec, itemoffset+LinkTargetIDOffset);
      target_index=IdeaLookup(IdeaDB, target_id);
      if (target_index!=-1) {
	 htarget=DmQueryRecord(IdeaDB, target_index);
	 ptarget=MemHandleLock(htarget);
	 for (i=0; i<IdeaNumItems(ptarget); i++) {
	    Word off=ItemOffset(ptarget, i);
	    
	    if (GetByte(ptarget, off+ItemTypeOffset)!=inverse_dir)
	       continue;
	    if (GetByte(ptarget,
			off+LinkTargetMindOffset(prec+itemoffset)))
	       continue;
	    if (GetLong(ptarget, off+LinkTargetIDOffset)==CurrentID) {
	       LinkInvOffset=off;
	       break;
	    }
	 };
	 MemHandleUnlock(htarget);
      };
   }

   /* set bothways checkbox */
   if (remote)
      CtlSetUsable(GetObjectPtr(LinkBothWaysCheckBox), false);
   else if ((!LinkInvOffset)&&GetLong(prec, itemoffset+LinkTargetIDOffset))
      CtlSetValue(GetObjectPtr(LinkBothWaysCheckBox), false);

   /* set label field */
   label=prec+itemoffset+LinkLabelOffset;
   labelfield=GetObjectPtr(LinkLabelField);
   FldInsert(labelfield, label, StrLen(label));
   if (LinkInvOffset) {
      CharPtr inv_label;
      
      ptarget=MemHandleLock(htarget);
      inv_label=ptarget+LinkInvOffset+LinkLabelOffset;
      if (StrCompare(label, inv_label)) {
	 FldInsert(labelfield, " | ", 3);
	 FldInsert(labelfield, inv_label, StrLen(inv_label));
      };
      MemHandleUnlock(htarget);
   };
   FldSetDirty(labelfield, false);

   /* set mindref list */
   list=GetObjectPtr(LinkTargetMindList);
   /*FIXME LstSetListChoices(list, NULL, 2+DmNumRecords(MindrefDB));
     LstSetDrawFunction(list, MindListDrawFunc);
   LstSetSelection(list, 0);
   if (remote) {
      int i;
      CharPtr mindref;

      mindref=prec+itemoffset+LinkTargetMindOffset(prec+itemoffset);

      for (i=0; i<DmNumRecords(MindrefDB); i++) {
	 VoidHand href;
	 Ptr pref;

	 href=DmQueryRecord(MindrefDB, i);
	 pref=MemHandleLock(href);
	 if (!StrCompare(pref, mindref)) {
	    LstSetSelection(list, i+1);
	    CtlSetLabel(GetObjectPtr(LinkTargetMindTrigger), pref);
	    break;
	 };
      };
      };*/

   /* set target field */
   if (!remote) {
      if (target_index!=-1) {
	 htarget=DmQueryRecord(IdeaDB, target_index);
	 ptarget=MemHandleLock(htarget);
	 targetfield=GetObjectPtr(LinkTargetField);
	 FldInsert(targetfield, ptarget+IdeaNameOffset,
		   StrLen(ptarget+IdeaNameOffset));
	 FldSetDirty(targetfield, false);
	 MemHandleUnlock(htarget);
      };
   } else {
      DmOpenRef othermind;
      Word otherindex;

      othermind=MindOpenFromMindref
	 (LstGetSelection(GetObjectPtr(LinkTargetMindList))-1);
      otherindex=IdeaLookup(othermind,
			    GetLong(prec, itemoffset+LinkTargetIDOffset));
      htarget=DmQueryRecord(othermind, otherindex);
      ptarget=MemHandleLock(htarget);
      targetfield=GetObjectPtr(LinkTargetField);
      FldInsert(targetfield, ptarget+IdeaNameOffset,
		StrLen(ptarget+IdeaNameOffset));
      FldSetDirty(targetfield,false);
      MemHandleUnlock(htarget);
      DmCloseDatabase(othermind);
   };
   
   MemHandleUnlock(hrec);
}

static void LinkFormCleanup()
{
   FieldPtr labelfield, targetfield;
   CharPtr newlabel;
   Word oldlabeloffset, oldlabellen, itemoffset;
   VoidHand hrec;
   Ptr prec;
   Boolean bothways, remote;
   Byte inv_dir;
   Word mindref_index;

   hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);
   
   /* set direction */
   if (CtlGetValue(GetObjectPtr(LinkChildPushButton))) {
      SetByte(prec, itemoffset+ItemTypeOffset, ChildType);
      inv_dir=ParentType;
   } else if (CtlGetValue(GetObjectPtr(LinkParentPushButton))) {
      SetByte(prec, itemoffset+ItemTypeOffset, ParentType);
      inv_dir=ChildType;
   } else {
      SetByte(prec, itemoffset+ItemTypeOffset, PeerType);
      inv_dir=PeerType;
   };
   MemHandleUnlock(hrec);
   
   /* set new label on this idea */
   labelfield=GetObjectPtr(LinkLabelField);
   if (FldDirty(labelfield)) {
      FldCompactText(labelfield);
      newlabel=FldGetTextPtr(labelfield);
      prec=MemHandleLock(hrec);
      oldlabeloffset=itemoffset+LinkLabelOffset;
      oldlabellen=StrLen(prec+oldlabeloffset);
      MemHandleUnlock(hrec);
      DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, false);
      DBRemove(IdeaDB, ActiveItemIdeaIndex, oldlabeloffset, oldlabellen);
      if (newlabel) {
	 Word len=0;
	 /* if there is a pipe, split the label there */
	 while ((newlabel[len]!=0)&&(newlabel[len]!='|'))
	    len++;
	 /* skip trailing spaces */
	 while ((len)&&(newlabel[len-1]==' '))
	    len--;
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, oldlabeloffset,
		  newlabel, len);
      }
      hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
   };

   mindref_index=LstGetSelection(GetObjectPtr(LinkTargetMindList));
   if (mindref_index) {
      remote=true;
      mindref_index--;
      bothways=false;
   } else {
      remote=false;
      bothways=CtlGetValue(GetObjectPtr(LinkBothWaysCheckBox));
   };
   /* set target on this idea */
   targetfield=GetObjectPtr(LinkTargetField);
   if (FldDirty(targetfield)) {
      DmOpenRef targetmind;
      SWord targetindex;
      CharPtr targetname;
      VoidHand htarget;
      Ptr ptarget;
      Word inv_size;

      
      /* delete old inverse link */
      if (!remote) {
	 prec=MemHandleLock(hrec);
	 targetindex=IdeaLookup(IdeaDB,
				GetLong(prec, itemoffset+LinkTargetIDOffset));
	 MemHandleUnlock(hrec);
	 if (targetindex!=-1) {
	    htarget=DmQueryRecord(IdeaDB, targetindex);
	    ptarget=MemHandleLock(htarget);
	    inv_size=GetWord(ptarget, LinkInvOffset+ItemSizeOffset);
	    MemHandleUnlock(htarget);
	    DBRemove(IdeaDB, targetindex, LinkInvOffset, inv_size);
	 };
      };

      /* find idea */
      if (remote) 
	 targetmind=MindOpenFromMindref(mindref_index);
      else
	 targetmind=IdeaDB;
      targetname=FldGetTextPtr(targetfield);
      targetindex=IdeaFindByName(targetmind, targetname);
      if (targetindex==-1) /* create idea */
	 targetindex=IdeaNew(targetmind, targetname);
      htarget=DmQueryRecord(targetmind, targetindex);
      ptarget=MemHandleLock(htarget);

      /* set link */
      prec=MemHandleLock(hrec);
      SetLong(prec, itemoffset+LinkTargetIDOffset,
	      GetLong(ptarget, IdeaIDOffset));
      MemHandleUnlock(hrec);

      if (remote) {
	 /* set mindref and title */
	 Word mindoffset, mindlen, titleoffset, titlelen;
	 /*VoidHand href;
	   Ptr pref;*/

	 prec=MemHandleLock(hrec);
	 mindoffset=itemoffset+LinkTargetMindOffset(prec+itemoffset);
	 mindlen=StrLen(prec+mindoffset);
	 titleoffset=mindoffset+mindlen+1;
	 titlelen=StrLen(prec+titleoffset);
	 MemHandleUnlock(hrec);
	 DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, true);
	 if (mindlen)
	    DBRemove(IdeaDB, ActiveItemIdeaIndex, mindoffset, mindlen);
	 titleoffset=mindoffset+1;
	 if (titlelen)
	    DBRemove(IdeaDB, ActiveItemIdeaIndex, titleoffset, titlelen);
	 /*FIXME href=DmQueryRecord(MindrefDB, mindref_index);
	 pref=MemHandleLock(href);
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, mindoffset, pref, StrLen(pref));
	 titleoffset+=StrLen(pref);
	 MemHandleUnlock(href);*/
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, titleoffset,
		  ptarget+IdeaNameOffset, StrLen(ptarget+IdeaNameOffset));
	 hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
      } else {
	 /* !remote; ensure that mindref and target are unset */
	 Word mindoffset, mindlen, titleoffset, titlelen;

	 prec=MemHandleLock(hrec);
	 mindoffset=itemoffset+LinkTargetMindOffset(prec+itemoffset);
	 mindlen=StrLen(prec+mindoffset);
	 titleoffset=mindoffset+mindlen+1;
	 titlelen=StrLen(prec+titleoffset);
	 MemHandleUnlock(hrec);
	 DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, true);
	 if (mindlen)
	    DBRemove(IdeaDB, ActiveItemIdeaIndex, mindoffset, mindlen);
	 titleoffset=mindoffset+1;
	 if (titlelen)
	    DBRemove(IdeaDB, ActiveItemIdeaIndex, titleoffset, titlelen);
	 hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
      };
	 
      MemHandleUnlock(htarget);
      
      /* make/set inverse link */
      if (bothways) {
	 Char zero[]={0,0,0};
	 CharPtr inv_label;

	 inv_label=FldGetTextPtr(labelfield);
	 if (inv_label) {
	    CharPtr tmp;
	    tmp=StrChr(inv_label, '|');
	    if (tmp) {
	       tmp++;
	       /* skip leading spaces */
	       while ((*tmp)==' ')
		  tmp++;
	       inv_label=tmp;
	    };
	 } else
	    inv_label="";

	 LinkInvOffset=MemHandleSize(htarget);	 
	 DBAppend(targetmind, targetindex, NULL, LinkLabelOffset+3);
	 htarget=DmGetRecord(targetmind, targetindex);
	 ptarget=MemHandleLock(htarget);
	 SetByte(ptarget, LinkInvOffset+ItemTypeOffset, inv_dir);
	 SetLong(ptarget, LinkInvOffset+LinkTargetIDOffset, CurrentID);
	 DmWrite(ptarget, LinkInvOffset+LinkLabelOffset, zero, 3);
	 SetWord(ptarget, LinkInvOffset+ItemSizeOffset,
		 LinkLabelOffset+3+StrLen(inv_label));
	 MemHandleUnlock(htarget);
	 DmReleaseRecord(targetmind, targetindex, true);
	 DBInsert(targetmind, targetindex, LinkInvOffset+LinkLabelOffset,
		  inv_label, StrLen(inv_label));
      };

      if (IdeaDB!=targetmind)
	 DmCloseDatabase(targetmind);
      
   } else if (bothways) {
      SWord targetindex;
      VoidHand htarget;
      Ptr ptarget;

      /* lookup inverse link */
      targetindex=IdeaLookup(IdeaDB,
			     GetLong(prec, itemoffset+LinkTargetIDOffset));
      if (targetindex!=-1) {
	 htarget=DmGetRecord(IdeaDB, targetindex);

	 /* set direction on inverse link */
	 ptarget=MemHandleLock(htarget);
	 SetByte(ptarget, LinkInvOffset+ItemTypeOffset, inv_dir);

	 /* set label on inverse link */
	 if (FldDirty(labelfield)) {
	    CharPtr oldlabel;
	    Word oldlabellen;
	    
	    newlabel=FldGetTextPtr(labelfield);
	    if (newlabel) {
	       CharPtr tmp;

	       tmp=StrChr(newlabel, '|');
	       if (tmp) {
		  /* skip the pipe and leading spaces */
		  do {
		     tmp++;
		  } while ((*tmp)==' ');
		  newlabel=tmp;
	       };
	    };
	    oldlabel=ptarget+LinkInvOffset+LinkLabelOffset;
	    oldlabellen=StrLen(oldlabel);

	    SetByte(ptarget, LinkInvOffset+ItemSizeOffset,
		    GetByte(ptarget, LinkInvOffset+ItemSizeOffset)
		    -oldlabellen+StrLen(newlabel));
	    
	    MemHandleUnlock(htarget);
	    DmReleaseRecord(IdeaDB, targetindex, true);
	    DBRemove(IdeaDB, targetindex, LinkInvOffset+LinkLabelOffset,
		     oldlabellen);
	    if (newlabel)
	       DBInsert(IdeaDB, targetindex, LinkInvOffset+LinkLabelOffset,
			newlabel, StrLen(newlabel));
	 } else {
	    MemHandleUnlock(htarget);
	    DmReleaseRecord(IdeaDB, targetindex, true);
	 };
      };
   };

   /* update size */
   SetWord(prec, itemoffset+ItemSizeOffset,
	   LinkSize(prec+itemoffset));
   IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);

   DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, true);
   
   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

static Boolean LinkCtlSelect(EventPtr e)
{
   Boolean handled=false;
   
   switch (e->data.ctlSelect.controlID) {
   case DoneButton:
      LinkFormCleanup();
      FrmReturnToForm(0);
      handled=true;
      break;

   case LinkBrowseButton: {
      Word index=BrowseForRecord();
      VoidHand hrec=DmQueryRecord(IdeaDB, index);
      Ptr prec=MemHandleLock(hrec);
      CharPtr text=prec+IdeaNameOffset;
      FieldPtr fld=GetObjectPtr(LinkTargetField);
      FldDelete(fld, 0, FldGetTextLength(fld));
      FldInsert(fld, text, StrLen(text));
      MemHandleUnlock(hrec);
      handled=true;
      break;
   };
   };

   return handled;
}

Boolean LinkFormHandleEvent(EventPtr e)
{
   Boolean handled=false;
   FormPtr frm;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      LinkFormInit();
      frm=FrmGetActiveForm();
      FrmDrawForm(frm);
      FrmSetFocus(frm, FrmGetObjectIndex(frm, LinkTargetField));
      handled=true;
      break;

   case frmCloseEvent:
      /* not normally called for Popup'd forms,
	 but i think it's called if the user launches another app,
	 so we need to unlock records, etc */
      LinkFormCleanup();

   case ctlSelectEvent:
      handled=LinkCtlSelect(e);
      break;

   case menuEvent:
      handled=EditMenuHandleEvent(e->data.menu.itemID);
      break;

   case popSelectEvent: {
      Ptr p=MindListGetItem(e->data.popSelect.selection);
      CtlSetLabel(GetObjectPtr(LinkTargetMindTrigger), p);
      MemPtrUnlock(p);
      /*FIXME if (e->data.popSelect.selection==1+DmNumRecords(MindrefDB)) {
	 MindrefIndex=-1;
	 FrmPopupForm(MindrefForm);
	 } else */if (e->data.popSelect.selection!=0) {
	 FldSetDirty(GetObjectPtr(LinkTargetField), true);
	 CtlSetUsable(GetObjectPtr(LinkBothWaysCheckBox), false);
      } else if (e->data.popSelect.selection==0) 
	 CtlSetUsable(GetObjectPtr(LinkBothWaysCheckBox), true);
      handled=true;
      break;
   }

   case mindrefDoneEvent: {
      ListPtr list=GetObjectPtr(LinkTargetMindList);
      ControlPtr trigger=GetObjectPtr(LinkTargetMindTrigger);
      Ptr p;

      FldSetDirty(GetObjectPtr(LinkTargetField), true);
      
      if (MindrefIndex==-1) {
	 LstSetSelection(list, 0);
	 p=MindListGetItem(0);
	 CtlSetLabel(trigger, p);
	 MemPtrUnlock(p);
	 CtlSetUsable(GetObjectPtr(LinkBothWaysCheckBox), true);
      } else {
	 /*LstSetListChoices(list, NULL, DmNumRecords(MindrefDB)+2);
	 LstSetSelection(list, MindrefIndex+1);
	 p=MindListGetItem(MindrefIndex+1);
	 CtlSetLabel(trigger, p);
	 MemPtrUnlock(p);
	 CtlSetUsable(GetObjectPtr(LinkBothWaysCheckBox), false);*/
      }
      break;
   };

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
