/* lmform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static VoidHand LMHrec;

void LMFollowLink()
{
   VoidHand hrec;
   Ptr prec;

   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   LinkFollow((LinkInfoPtr)(prec+ItemOffset(prec, ActiveItemIndex)
			    +LMDataOffset));
   MemHandleUnlock(hrec);
}

static void LMFormInit()
{
   Ptr prec;
   Word offset, len;

   LMHrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(LMHrec);
   offset=ItemOffset(prec, ActiveItemIndex)+LMTextOffset;
   len=StrLen(prec+offset)+1;
   MemHandleUnlock(LMHrec);

   FldSetText(GetObjectPtr(LMTextField), LMHrec, offset, len);
}

static void LMFormCleanup()
{
   FieldPtr fld=GetObjectPtr(LMTextField);
   Boolean dirty=FldDirty(fld);
   Ptr prec;
   Word size;

   FldCompactText(fld);
   size=LMTextOffset+FldGetTextLength(fld)+1;
   FldSetTextHandle(fld, NULL);

   /* update size */
   prec=MemHandleLock(LMHrec);
   SetWord(prec, ItemOffset(prec, ActiveItemIndex)+ItemSizeOffset, size);
   IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);
   MemHandleUnlock(LMHrec);
   DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, dirty);

   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

Boolean LMFormHandleEvent(EventPtr e)
{
   Boolean handled=false;
   FormPtr frm;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      LMFormInit();
      frm=FrmGetActiveForm();
      FrmDrawForm(frm);
      FrmSetFocus(frm, FrmGetObjectIndex(frm, LMTextField));
      handled=true;
      break;

   case frmCloseEvent:
      LMFormCleanup();
      break;

   case ctlSelectEvent:
      switch (e->data.ctlSelect.controlID) {
      case DoneButton:
	 LMFormCleanup();
	 FrmReturnToForm(0);
	 handled=true;
	 break;
      };
      break;

   case menuEvent:
      handled=EditMenuHandleEvent(e->data.menu.itemID);
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
