/* mainform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "linkaware.h"
#include "thoughtstream.h"

signed char MainDisplayModes[][NumItemTypes+1]={
   {1, -1, 2, 1, 1, 1, 1, 1, 1}, /* normal */
   {1,  1, 1, 1, 1, 1, 1, 1, 1}, /* unsorted */
   {1,  1, 1, 1, 1, 1, 1, 1, 1}, /* custom */
};
#define NumDisplayModes 3

typedef struct {
   int priority;
   Word ideaindex;
   SWord itemindex;
   Word indention;
   Boolean used;
} RowInfo;   


static Char MainFormTitle[32];
static RowInfo* MainRowInfo;
static SWord TopVisibleRow;
static Word MainYMin=15;
static Word MainYMax=146;
static int MainSelection;

static void MainSelectItem(Word row);
static void MainScroll(Word key);

static RowInfo* RowInfoNew(Word ideaindex)
{
   RowInfo* result;
   int i, j, numitems, minpri=0, maxpri=0, pos=0;
   signed char* dispmode=MainDisplayModes[prefs.display_mode];
   VoidHand hidea;
   Ptr pidea;

   hidea=DmQueryRecord(IdeaDB, ideaindex);
   pidea=MemHandleLock(hidea);

   /* count displayed items */
   numitems=1; /* start at 1 for title */
   for (i=0; i<IdeaNumItems(pidea); i++) {
      Char itemtype=GetByte(pidea, ItemOffset(pidea, i)+ItemTypeOffset);
      int pri;
      if (itemtype<NumItemTypes)
	 pri=dispmode[itemtype];
      else
	 pri=dispmode[NumItemTypes];
      if (pri)
	 numitems++;
      if (pri<minpri)
	 minpri=pri;
      if (pri>maxpri)
	 maxpri=pri;
   }
   result=MemPtrNew(numitems*sizeof(RowInfo));

   for (i=minpri; i<=maxpri; i++) {
      if (i==0) {
	 /* add extra entry for title */
	 result[pos].priority=0;
	 result[pos].ideaindex=ideaindex;
	 result[pos].itemindex=-1;
	 result[pos].indention=0;
	 pos++;
	 continue;
      };
      for (j=0; j<IdeaNumItems(pidea); j++) {
	 Byte itemtype=GetByte(pidea, ItemOffset(pidea,j)+ItemTypeOffset);
	 if (itemtype>NumItemTypes)
	    itemtype=NumItemTypes;
	 if (dispmode[itemtype]==i) {
	    result[pos].priority=i;
	    result[pos].ideaindex=ideaindex;
	    result[pos].itemindex=j;
	    result[pos].indention=0;
	    pos++;
	 };
      };
   };
   MemHandleUnlock(hidea);
   return result;
}

static int MaxRows()
{
   FontID oldfont;
   int result;

   oldfont=FntSetFont(prefs.font);
   result=(MainYMax-MainYMin)/FntLineHeight();
   FntSetFont(oldfont);
   return result;
}

static int NumRows()
{
   return MemPtrSize(MainRowInfo)/sizeof(RowInfo);
}

/* When this function is called, the 3 Mind DB's should be
   opened and CurrentID should be set to the idea to load.
*/
void MainFormInit(FormPtr frm)
{
   UInt cardno;
   LocalID dbid;

   /* find the current idea */
   CurrentIndex=IdeaLookup(IdeaDB, CurrentID);
   if (CurrentIndex==dmMaxRecordIndex)
      FrmGotoForm(MindForm);

   /* push history entry */
   LinkHistoryPush((LinkInfoPtr)TSLinkNew(IdeaDB, CurrentIndex));

   /* update back/forward buttons */
   if (LinkHistoryBackCheck())
      CtlSetUsable(GetObjectPtr(MainBackButton), true);
   else
      CtlSetUsable(GetObjectPtr(MainBackButton), false);
   if (LinkHistoryForwardCheck())
      CtlSetUsable(GetObjectPtr(MainForwardButton), true);
   else
      CtlSetUsable(GetObjectPtr(MainForwardButton), false);

   /* set the form title to the mind name */
   DmOpenDatabaseInfo(IdeaDB, &dbid, NULL, NULL, &cardno, NULL);
   DmDatabaseInfo(cardno, dbid, MainFormTitle, NULL, NULL, NULL, NULL,
		  NULL, NULL, NULL, NULL, NULL, NULL);
   FrmSetTitle(frm, MainFormTitle);

   /* save current position in prefs */
   StrCopy(prefs.last_mind, MainFormTitle);
   prefs.lastID=CurrentID;

   /* construct the list of rows */
   if (MainRowInfo) MemPtrFree(MainRowInfo);
   MainRowInfo=RowInfoNew(CurrentIndex);

   MainSelection=-1;
   TopVisibleRow=0;
   MainScroll(0);
}   

static LocalID DrawIcon(ULong creator, int x, int y)
{
   DmOpenRef otherapp;
   VoidHand hicon;
   BitmapPtr picon;
   Boolean icondrawn=false;
   UInt cardno;
   LocalID dbid=0;

   if (creator) {
      otherapp=DmOpenDatabaseByTypeCreator('appl', creator,
					   dmModeReadOnly);
      if (otherapp) {
	 DmOpenDatabaseInfo(otherapp, &dbid, NULL, NULL, &cardno, NULL);
	 hicon=DmGet1Resource('tAIB', 1001);
	 if (hicon) {
	    picon=(BitmapPtr)MemHandleLock(hicon);
	    WinDrawBitmap(picon, x,y);
	    MemHandleUnlock(hicon);
	    DmReleaseResource(hicon);
	    icondrawn=true;
	 };
	 DmCloseDatabase(otherapp);
      };
   };

   if (!icondrawn) {
      FntSetFont(boldFont);
      WinDrawChars("?", 1, x+5, y);
      FntSetFont(prefs.font);
   }

   /* sort of a cheat here; this is just for convenience for
      drawing searches with no data */
   return dbid;
}

static void RowDraw(Word row)
{
   int x,y;
   FontID oldfont;
   VoidHand hidea, hstr;
   Ptr pidea, pstr;
   RectangleType bounds;

   if (row>=TopVisibleRow+MaxRows())
      return;
   
   oldfont=FntSetFont(prefs.font);
   x=SmallIconWidth*MainRowInfo[row].indention;
   y=MainYMin+(row-TopVisibleRow)*FntLineHeight();
   RctSetRectangle(&bounds, 0,y, 160, FntLineHeight());
   WinEraseRectangle(&bounds, 0);

   hidea=DmQueryRecord(IdeaDB, MainRowInfo[row].ideaindex);
   pidea=MemHandleLock(hidea);
   
   if (MainRowInfo[row].priority==0) {
      /* title */
      CharPtr title;
      
      FntSetFont(boldFont);
      title=pidea+IdeaNameOffset;
      WinDrawChars(title, StrLen(title), x+SmallIconWidth,y);
   } else if (row>=TopVisibleRow) {
      Char noteicon[2]={symbolNote, 0};
      CharPtr text;
      Word textlen, itemoff;
      Byte type;
     
      itemoff=ItemOffset(pidea, MainRowInfo[row].itemindex);
      type=GetByte(pidea, itemoff+ItemTypeOffset);
      switch (type) {
      case NoteType:
	 FntSetFont(symbolFont);
	 WinDrawChars(noteicon, 1, x+5,y);
	 FntSetFont(prefs.font);
	 text=pidea+itemoff+NoteTextOffset;
	 if (StrChr(text, '\n'))
	    textlen=StrChr(text,'\n')-text;
	 else
	    textlen=StrLen(text);
	 WinDrawChars(text, textlen, x+SmallIconWidth,y);
	 break;

      case URLType:
	 text=pidea+itemoff+URLTitleOffset;
	 if (!(*text))
	    text=pidea+itemoff+URLAddressOffset(pidea+itemoff);
	 WinDrawChars(text, StrLen(text), x+SmallIconWidth, y);
	 break;

      case SearchType: {
	 ULong creator;
	 LocalID dbid;
	 
	 creator=GetLong(pidea, itemoff+SearchCreatorOffset);
	 dbid=DrawIcon(creator, x, y);
	 
	 text=pidea+itemoff+SearchTextOffset;
	 if (*text)
	    WinDrawChars(text, StrLen(text), x+SmallIconWidth, y);
	 else {
	    if (dbid) {
	       Char othername[33];
	       DmDatabaseInfo(0, dbid, othername, NULL, NULL, NULL,
			      NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	       WinDrawChars(othername, StrLen(othername), x+SmallIconWidth,y);
	    } else {
	       hstr=DmGetResource('tSTR', UnknownAppString);
	       pstr=MemHandleLock(hstr);
	       WinDrawChars(pstr, StrLen(pstr), x+SmallIconWidth, y);
	       MemHandleUnlock(hstr);
	    };
	 };
	 break;
      };

      case LMType:
	 DrawIcon(GetLong(pidea, itemoff+LMDataOffset), x,y);
	 text=pidea+itemoff+LMTextOffset;
	 WinDrawChars(text, StrLen(text), x+SmallIconWidth, y);
	 break;

      case FieldType:
	 /* FIXME: RowDraw FieldType */
	 WinDrawChars("field", 5, x+SmallIconWidth, y);
	 break;

      case ParentType:
      case ChildType:
      case PeerType: {
	 Char arrows[]={symbolUpArrow, symbolDownArrow, symbolRightArrow};
	 ULong target_id;
	 
	 FntSetFont(symbolFont);
	 WinDrawChars(&arrows[type-ParentType],
		      1, x+2, y);
	 FntSetFont(prefs.font);

	 target_id=GetLong(pidea, itemoff+LinkTargetIDOffset);
	 if (!target_id) {
	    hstr=DmGetResource('tSTR', UnsetTargetString);
	    pstr=MemHandleLock(hstr);
	    WinDrawChars(pstr, StrLen(pstr), x+SmallIconWidth, y);
	    MemHandleUnlock(hstr);
	 } else {
	    /* local link */
	    Word targetindex;
	    VoidHand htarget=0;
	    Ptr ptarget;
	    CharPtr label, targettitle;
	    Byte remote=(pidea+itemoff+LinkTargetMindOffset(pidea+itemoff))[0];

	    if (remote)
	       targettitle=pidea+itemoff+LinkTargetTitleOffset(pidea+itemoff);
	    else {
	       targetindex=IdeaLookup(IdeaDB, target_id);
	       htarget=DmQueryRecord(IdeaDB, targetindex);
	       ptarget=MemHandleLock(htarget);
	       targettitle=ptarget+IdeaNameOffset;
	    }
	    label=pidea+itemoff+LinkLabelOffset;
	    if (*label) {
	       CharPtr text, format="%s: %s";
	       
	       text=MemPtrNew(StrLen(label)+StrLen(targettitle)
			      +StrLen(format));
	       StrPrintF(text, format, label, targettitle);
	       WinDrawChars(text, StrLen(text), x+SmallIconWidth, y);
	       MemPtrFree(text);
	    } else 
	       WinDrawChars(targettitle, StrLen(targettitle),
			    x+SmallIconWidth, y);
	    if (!remote)
	       MemHandleUnlock(htarget);
	 };
	 break;
      };
	 
      default:
	 FntSetFont(boldFont);
	 WinDrawChars("?", 1, x+5, y);
	 FntSetFont(prefs.font);
	 hstr=DmGetResource('tSTR', UnknownItemString);
	 pstr=MemHandleLock(hstr);
	 WinDrawChars(pstr, StrLen(pstr), x+SmallIconWidth,y);
	 MemHandleUnlock(hstr);
	 break;
      };
   };
   if (MainSelection==row) {
      MainSelection=-1;
      MainSelectItem(row);
   }
   FntSetFont(oldfont);
   MemHandleUnlock(hidea);
   return;
}

void MainFormDraw()
{
   Word row;

   row=TopVisibleRow;
   while ((row<NumRows())&&(row<TopVisibleRow+MaxRows())) {
      RowDraw(row++);
   }
}

static void MainSelectItem(Word row)
{
   int lineheight, split;
   RectangleType rect;
   FontID oldfont;
   
   if (MainSelection==row)
      return;

   oldfont=FntSetFont(prefs.font);
   lineheight=FntLineHeight();
   FntSetFont(oldfont);

   if (MainSelection!=-1) {
      split=SmallIconWidth
	 *(MainRowInfo[MainSelection+TopVisibleRow].indention+1);
      RctSetRectangle(&rect, 0, MainSelection*lineheight+MainYMin,
		      split, lineheight);
      WinInvertRectangle(&rect, 0);
   };

   split=SmallIconWidth*(MainRowInfo[row+TopVisibleRow].indention+1);
   RctSetRectangle(&rect, 0, row*lineheight+MainYMin, split, lineheight);
   WinInvertRectangle(&rect, 0);

   MainSelection=row;
}

static Boolean PenTracking;
static RectangleType TrackingRect;
static Boolean TrackingRectInverted;

static Boolean MainPenDown(EventPtr e)
{
   int row, lineheight;
   FontID oldfont;
   Word split;
   
   if ((e->screenY<MainYMin)
       ||(e->screenY>MainYMax))
      return false;

   row=e->screenY-MainYMin;
   oldfont=FntSetFont(prefs.font);
   lineheight=FntLineHeight();
   row/=lineheight;
   FntSetFont(oldfont);

   if (row>=NumRows())
      return false;
   
   split=SmallIconWidth*(MainRowInfo[row].indention+1);
   if (e->screenX<split)
      RctSetRectangle(&TrackingRect, 0, row*lineheight+MainYMin,
		      split, lineheight);
   else
      RctSetRectangle(&TrackingRect, split, row*lineheight+MainYMin,
		      160-split, lineheight);
   
   WinInvertRectangle(&TrackingRect, 0);
   TrackingRectInverted=true;
   PenTracking=true;
   return true;
}

static void MainNew(Word selection)
{
   Word item_index=0;
   
   /* WARNING: keep this sync'd with MainNewList in thoughtstream.rcp */
   switch (selection) {
   case 0: /* field */
      item_index=ItemNew(IdeaDB, CurrentIndex, FieldType);
      break;

   case 1: /* linkmaster */
      LinkRequest((LinkInfoPtr)TSLinkNew(IdeaDB, CurrentIndex));
      return;

   case 2: /* search */
      item_index=ItemNew(IdeaDB, CurrentIndex, SearchType);
      break;

   case 3: /* url */
      item_index=ItemNew(IdeaDB, CurrentIndex, URLType);
      break;

   case 4: /* note */
      item_index=ItemNew(IdeaDB, CurrentIndex, NoteType);
      break;

   case 5: /* link */
      item_index=ItemNew(IdeaDB, CurrentIndex, ChildType);
      break;

   case -1: /* nothing */
      return;
   };
   MainFormInit(FrmGetActiveForm());
   ItemActivate(CurrentIndex, item_index, ActivateEdit);
}

static void MainSetMode(SWord selection)
{
   if (selection==-1)
      return;
   prefs.display_mode=selection;
   MainFormInit(FrmGetActiveForm());
   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

static Boolean MainPenMove(EventPtr e)
{
   if (!PenTracking)
      return false;
   if (RctPtInRectangle(e->screenX, e->screenY, &TrackingRect)
       ^TrackingRectInverted) {
      WinInvertRectangle(&TrackingRect, 0);
      TrackingRectInverted=!TrackingRectInverted;
   };
   return true;
}

static Boolean MainPenUp(EventPtr e)
{
   int row, lineheight;
   FontID oldfont;
   
   if (!PenTracking)
      return false;
   if (TrackingRectInverted) 
      WinInvertRectangle(&TrackingRect, 0);
   TrackingRectInverted=PenTracking=false;
   if (!RctPtInRectangle(e->screenX, e->screenY, &TrackingRect)) 
      return true;

   row=e->screenY-MainYMin;
   oldfont=FntSetFont(prefs.font);
   lineheight=FntLineHeight();
   row/=lineheight;
   FntSetFont(oldfont);

   /* do something with row here */
   if (TrackingRect.topLeft.x==0) /* left column */
      MainSelectItem(row);
   else
      ItemActivate(MainRowInfo[row+TopVisibleRow].ideaindex,
		   MainRowInfo[row+TopVisibleRow].itemindex,
		   ActivateDefault);

   return true;
}

static void MainMenuEvent(Word item)
{
   switch (item) {
   case MainRecordNewOpenMind:
      MindClose();
      FrmGotoForm(MindForm);
      break;

   case MainRecordPublishLink:
      LinkPublish((LinkInfoPtr)TSLinkNew(IdeaDB, CurrentIndex));
      break;

   case MainRecordSetHome: {
      UInt cardno;
      LocalID dbid;
      
      DmOpenDatabaseInfo(IdeaDB, &dbid, NULL, NULL, &cardno, NULL);
      DmDatabaseInfo(cardno, dbid, prefs.home_mind, NULL, NULL, NULL,
		     NULL, NULL, NULL, NULL, NULL, NULL, NULL);
      prefs.homeID=CurrentID;
      break;
   };

   case MainGoBack:
      LinkHistoryBack();
      break;

   case MainGoForward:
      LinkHistoryForward();
      break;

   case MainOptionsPrefs:
      FrmPopupForm(PrefsForm);
      break;
      
   case MainOptionsAbout:
      FrmAlert(AboutAlert);
      break;
   };
}

static void MainScroll(Word key)
{
   ControlPtr ctl;
   
   /* adjust position */
   switch (key) {
   case pageUpChr:
      TopVisibleRow-=(MaxRows()-1);
      break;

   case pageDownChr:
      TopVisibleRow+=(MaxRows()-1);
      break;

   case 0: /* don't change, just update scroll buttons */
      break;
   };

   /* clamp range */
   if (TopVisibleRow>(NumRows()-MaxRows()))
      TopVisibleRow=NumRows()-MaxRows();
   if (TopVisibleRow<0)
      TopVisibleRow=0;

   /* update scrollers and display */
   ctl=GetObjectPtr(MainScrollUpButton);
   CtlSetUsable(ctl, false);
   if (TopVisibleRow==0)
      CtlSetLabel(ctl, "\003");
   else
      CtlSetLabel(ctl, "\001");
   CtlSetUsable(ctl, true);

   ctl=GetObjectPtr(MainScrollDownButton);
   CtlSetUsable(ctl, false);
   if (TopVisibleRow+MaxRows()>=NumRows())
      CtlSetLabel(ctl, "\004");
   else
      CtlSetLabel(ctl, "\002");
   CtlSetUsable(ctl, true);

   if (key)
      FrmUpdateForm(MainForm, frmRedrawUpdateCode);   
}

Boolean MainFormHandleEvent (EventPtr e)
{
   Boolean handled = false;
    
   CALLBACK_PROLOGUE

      switch (e->eType) {
      case frmOpenEvent:
	 MainFormInit(FrmGetActiveForm());
	 /* fall through */
	 
      case frmUpdateEvent:
	 FrmDrawForm(FrmGetActiveForm());
	 MainFormDraw();
	 handled=true;
	 break;

      case frmCloseEvent:
	 if (MainRowInfo)
	    MemPtrFree(MainRowInfo);
	 MainRowInfo=0;
	 break;

      case frmGotoEvent: {
	 UInt cardno;
	 LocalID dbid;
	 Char dbname[33];

	 /* un-kludge db info */
	 cardno=e->data.frmGoto.recordNum;
	 dbid=e->data.frmGoto.matchLen;
	 dbid<<=16;
	 dbid|=e->data.frmGoto.matchPos;

	 DmDatabaseInfo(cardno, dbid, dbname, NULL, NULL, NULL, NULL,
			NULL, NULL, NULL, NULL, NULL, NULL);
	 MindClose();
	 if (MindOpen(dbname)) {
	    CurrentID=e->data.frmGoto.matchCustom;
	    WinEraseWindow();
	    MainFormInit(FrmGetActiveForm());
	    FrmDrawForm(FrmGetActiveForm());
	    MainFormDraw();
	    handled=true;
	    break;
	 } else
	    ErrDisplay("Can't happen");
	 break;
      };
	 

      case menuEvent:
	 MenuEraseStatus(NULL);

	 MainMenuEvent(e->data.menu.itemID);
	 handled = true;
	 break;

      case ctlSelectEvent:
	 switch (e->data.ctlSelect.controlID) {
	    /*case MainNewButton:
	    MainNew();
	    break;*/

	 case MainDetailsButton:
	    if (MainSelection==-1) break;
	    ItemActivate(MainRowInfo[MainSelection+TopVisibleRow].ideaindex,
			 MainRowInfo[MainSelection+TopVisibleRow].itemindex,
			 ActivateEdit);
	    break;

	 case MainDeleteButton:
	    if (MainSelection==-1) break;
	    if (MainRowInfo[MainSelection+TopVisibleRow].itemindex==-1) {
	       /* title - delete idea */
	       if (FrmCustomAlert(ConfirmDeleteAlert, "this idea", 0,0)>0)
		  break;
	       IdeaDelete(IdeaDB,
			  MainRowInfo[MainSelection+TopVisibleRow].ideaindex);

	       /* try to go to another idea; if LM is not installed,
		  then just drop back to idea 0 */
	       if (!LinkHistoryBack()) {
		  VoidHand hrec;
		  Ptr prec;

		  hrec=DmQueryRecord(IdeaDB, 0);
		  prec=MemHandleLock(hrec);
		  CurrentID=GetLong(prec, IdeaIDOffset);
		  MemHandleUnlock(hrec);
		  WinEraseWindow();
		  MainFormInit(FrmGetActiveForm());
		  FrmDrawForm(FrmGetActiveForm());
		  MainFormDraw();
	       };
	       break;
	    }
	    /* this will have to be fixed for i18n */
	    if (FrmCustomAlert(ConfirmDeleteAlert, "this item",0,0)>0) break;
	    ItemDelete(IdeaDB,
		       MainRowInfo[MainSelection+TopVisibleRow].ideaindex,
		       MainRowInfo[MainSelection+TopVisibleRow].itemindex);
	    MainFormInit(FrmGetActiveForm());
	    WinEraseWindow();
	    FrmUpdateForm(MainForm, frmRedrawUpdateCode);
	    break;

	 case MainHomeButton:
	    if (!prefs.homeID) {
	       FrmAlert(HomeNotSetAlert);
	       break;
	    };
	    MindClose();
	    MindOpen(prefs.home_mind);
	    CurrentID=prefs.homeID;
	    MainFormInit(FrmGetActiveForm());
	    WinEraseWindow();
	    FrmUpdateForm(MainForm, frmRedrawUpdateCode);
	    break;

	 case MainBackButton:
	    LinkHistoryBack();
	    break;

	 case MainForwardButton:
	    LinkHistoryForward();
	    break;
	 };
	 break;

      case ctlRepeatEvent:
	 switch (e->data.ctlRepeat.controlID) {
	 case MainScrollUpButton:
	    MainScroll(pageUpChr);
	    break;

	 case MainScrollDownButton:
	    MainScroll(pageDownChr);
	    break;
	 };
	 break;

      case popSelectEvent:
	 switch (e->data.popSelect.listID) {
	 case MainNewList:
	    MainNew(e->data.popSelect.selection);
	    handled=true;
	    break;

	 case MainModeList:
	    MainSetMode(e->data.popSelect.selection);
	    /*handled=true;*/
	    break;
	 };
	 break;

      case penDownEvent:
	 handled=MainPenDown(e);
	 break;
	 
      case penMoveEvent:
	 handled=MainPenMove(e);
	 break;
	 
      case penUpEvent:
	 handled=MainPenUp(e);
	 break;

      case keyDownEvent:
	 switch (e->data.keyDown.chr) {
	 case pageUpChr:
	 case pageDownChr:
	    MainScroll(e->data.keyDown.chr);
	    break;
	 };
	 break;
	 

      default:
	 break;
      }

   CALLBACK_EPILOGUE

      return handled;
}
