/* mindform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static Boolean MindNew(CharPtr name)
{
   VoidHand hrec, happinfo;
   Ptr prec, pappinfo;
   LocalID appinfoid, dbid;
   UInt cardno;

   if ((!name)||(!name[0]))
      return false;
   
   MindClose();
   
   IdeaDB=DBCreate('ThSt', 'MIND', name);
   if (!IdeaDB) return false;

   /*StrCopy(buf, name);
   StrCat(buf, "S");
   SyncDB=DBCreate('ThSt', 'SYNC', buf);
   if (!SyncDB) return false;

   StrCopy(buf, name);
   StrCat(buf, "M");
   MindrefDB=DBCreate('ThSt', 'MREF', buf);
   if (!MindrefDB) return false;*/

   IdeaNew(IdeaDB, name);

   /* set appinfo for default id */
   hrec=DmQueryRecord(IdeaDB, 0);
   prec=MemHandleLock(hrec);
   happinfo=DmNewHandle(IdeaDB, 4);
   pappinfo=MemHandleLock(happinfo);
   DmWrite(pappinfo, 0, prec, 4);
   CurrentID=GetLong(prec, 0);
   MemHandleUnlock(happinfo);
   MemHandleUnlock(hrec);
   appinfoid=MemHandleToLocalID(happinfo);
   DmOpenDatabaseInfo(IdeaDB, &dbid, NULL, NULL, &cardno, NULL);
   DmSetDatabaseInfo(cardno, dbid, NULL, NULL, NULL, NULL,
		     NULL, NULL, NULL, &appinfoid, NULL, NULL, NULL);
   
   return true;
}

void MindClose()
{
   if (IdeaDB) DmCloseDatabase(IdeaDB);
   IdeaDB=NULL;
   /*if (SyncDB) DmCloseDatabase(SyncDB);
     SyncDB=NULL;
     if (MindrefDB) DmCloseDatabase(MindrefDB);
     MindrefDB=NULL;*/
}

static void MindOpenListDraw(UInt item, RectanglePtr bounds,
			     SysDBListItemType* dbinfo)
{   
   CALLBACK_PROLOGUE;
   WinDrawChars(dbinfo[item].name, StrLen(dbinfo[item].name),
		bounds->topLeft.x, bounds->topLeft.y);
   CALLBACK_EPILOGUE;
}

static Handle hMindFormDBInfo;

static void MindFormInit(FormPtr frm)
{
   Word dbcount;
   SysDBListItemType* pDBInfo;
   ListPtr lst;
   
   /* populate list with all ThSt.MIND DB's */
   SysCreateDataBaseList('MIND', 'ThSt', &dbcount, &hMindFormDBInfo, false);
   if (dbcount) {
      pDBInfo=MemHandleLock((VoidHand)hMindFormDBInfo);
      lst=GetObjectPtr(MindOpenList);
      LstSetListChoices(lst, (CharPtr*)pDBInfo, dbcount);
      LstSetDrawFunction(lst, (ListDrawDataFuncPtr)MindOpenListDraw);
   } else {
      hMindFormDBInfo=NULL;
      LstSetListChoices(GetObjectPtr(MindOpenList), 0, 0);
   };
}

static void MindFormCleanup()
{
   if (hMindFormDBInfo) { 
      MemHandleUnlock((VoidHand)hMindFormDBInfo);
      MemHandleFree((VoidHand)hMindFormDBInfo);
   };
}

static Boolean MindFormOpen()
{
   SysDBListItemType* pDBInfo;
   Word selection;
   ListPtr lst;
   Boolean retval;

   lst=GetObjectPtr(MindOpenList);
   selection=LstGetSelection(lst);
   if (selection==-1)
      return false;
 
   MindClose();

   pDBInfo=MemHandleLock((VoidHand)hMindFormDBInfo);
   retval=MindOpen(pDBInfo[selection].name);
   
   MemHandleUnlock((VoidHand)hMindFormDBInfo);
   return retval;
}

DmOpenRef MindOpenFromMindref(Word index)
{
   /* FIXME */
   /*   VoidHand hrec;
   Ptr prec;
   CharPtr p;
   DmOpenRef retval=0;

   hrec=DmQueryRecord(MindrefDB, index);
   prec=MemHandleLock(hrec);
   p=prec+StrLen(prec)+2;
   while (p-prec<MemHandleSize(hrec)) {
      if (p[0]==PalmDBMindref) {
	 LocalID dbid;

	 dbid=DmFindDatabase(0, p+1);
	 if (dbid) {
	    retval=DmOpenDatabase(0, dbid, dmModeReadWrite);
	    if (retval)
	       break;
	 }
      } else
	 p+=StrLen(p+1)+2;
   }
   MemHandleUnlock(hrec);
   return retval;*/
}

Boolean MindOpen(CharPtr name)
{
   LocalID dbID, appinfoid;
   Ptr pappinfo;

   dbID=DmFindDatabase(0, name);
   if (!dbID) goto exit;
   IdeaDB=DmOpenDatabase(0, dbID, dmModeReadWrite);
   if (!IdeaDB) goto exit;

   /*StrCopy(tmp, name);
   StrCat(tmp, "S");
   dbID=DmFindDatabase(0, tmp);
   if (!dbID) goto exit;
   SyncDB=DmOpenDatabase(0, dbID, dmModeReadWrite);
   if (!SyncDB) goto exit;

   StrCopy(tmp, name);
   StrCat(tmp, "M");
   dbID=DmFindDatabase(0, tmp);
   if (!dbID) goto exit;
   MindrefDB=DmOpenDatabase(0, dbID, dmModeReadWrite);
   if (!MindrefDB) goto exit;*/

   /* read appinfo for default idea */
   appinfoid=DmGetAppInfoID(IdeaDB);
   pappinfo=MemLocalIDToLockedPtr(appinfoid, 0);
   CurrentID=GetLong(pappinfo, 0);
   MemPtrUnlock(pappinfo);

   return true;
   
 exit:
   IdeaDB=0;
   /*SyncDB=0;
     MindrefDB=0;*/
   return false;
}

static void MindFormDelete()
{
   ListPtr lst;
   Word selection;
   SysDBListItemType* pDBInfo;

   if (!hMindFormDBInfo) return;
   pDBInfo=MemHandleLock((VoidHand)hMindFormDBInfo);
   lst=GetObjectPtr(MindOpenList);
   selection=LstGetSelection(lst);
   if (selection<0) return; 
   if (FrmCustomAlert(ConfirmDeleteAlert, pDBInfo[selection].name,0,0)==0) {
      Char name[33];
      LocalID dbID;
      
      /* delete the DBs */
      DmDeleteDatabase(pDBInfo[selection].cardNo, pDBInfo[selection].dbID);

      StrCopy(name, pDBInfo[selection].name);
      StrCat(name, "S");
      dbID=DmFindDatabase(0, name);
      DmDeleteDatabase(0, dbID);

      StrCopy(name, pDBInfo[selection].name);
      StrCat(name, "M");
      dbID=DmFindDatabase(0, name);
      DmDeleteDatabase(0, dbID);

      MemHandleUnlock((VoidHand)hMindFormDBInfo);
      MindFormCleanup();
      MindFormInit(FrmGetActiveForm());
      FrmDrawForm(FrmGetActiveForm());
   } else
      MemHandleUnlock((VoidHand)hMindFormDBInfo);
}

Boolean MindFormHandleEvent (EventPtr e)
{
   Boolean handled = false;
   FormPtr frm;
    
   CALLBACK_PROLOGUE

      switch (e->eType) {
      case frmOpenEvent:
	 frm = FrmGetActiveForm();
	 MindFormInit(frm);
	 FrmDrawForm(frm);
	 FrmSetFocus(frm, FrmGetObjectIndex(frm, MindNewField));
	 handled = true;
	 break;

      case frmCloseEvent:
	 MindFormCleanup();
	 break;
	 
      case menuEvent:
	 MenuEraseStatus(NULL);

	 switch(e->data.menu.itemID) {
	 }

	 handled = true;
	 break;

      case ctlSelectEvent:
	 switch(e->data.ctlSelect.controlID) {
	 case MindNewButton:
	    if (MindNew(FldGetTextPtr(GetObjectPtr(MindNewField))))
	       FrmGotoForm(MainForm);
	    handled=true;
	    break;

	 case MindOpenButton:
	    if (MindFormOpen())
	       FrmGotoForm(MainForm);
	    handled=true;
	    break;

	 case MindDelButton:
	    MindFormDelete();
	    handled=true;
	    break;
	 }
	 break;

      default:
	 break;
      }

   CALLBACK_EPILOGUE

      return handled;
}
