/* mindrefform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

/* MindrefIndex must be set before calling FrmPopupForm(MindrefForm) */
SWord MindrefIndex=-1;

static Handle hMindrefFormDBInfo;

static void MindrefMindListDraw(UInt item, RectanglePtr bounds,
				SysDBListItemType* dbinfo)
{
   CALLBACK_PROLOGUE;
   WinDrawChars(dbinfo[item].name, StrLen(dbinfo[item].name),
		bounds->topLeft.x, bounds->topLeft.y);
   CALLBACK_EPILOGUE;
}

static void MindrefFormInit()
{
   SysDBListItemType* pDBInfo;
   ListPtr lst;
   Word dbcount;

   /* populate list with all ThSt.MIND DB's */
   SysCreateDataBaseList('MIND', 'ThSt', &dbcount, &hMindrefFormDBInfo, false);
   if (dbcount) {
      pDBInfo=MemHandleLock((VoidHand)hMindrefFormDBInfo);
      lst=GetObjectPtr(MindrefMindList);
      LstSetListChoices(lst, (CharPtr*)pDBInfo, dbcount);
      LstSetDrawFunction(lst, (ListDrawDataFuncPtr)MindrefMindListDraw);
   } else {
      hMindrefFormDBInfo=NULL;
      LstSetListChoices(GetObjectPtr(MindrefMindList), 0,0);
   };

   /* set field */
   /*FIXME if (MindrefIndex!=-1) {
      VoidHand hrec;
      Ptr prec;
      FieldPtr fld;
      FieldAttrType attr;

      hrec=DmQueryRecord(MindrefDB, MindrefIndex);
      prec=MemHandleLock(hrec);
      fld=GetObjectPtr(MindrefRefField);
      FldInsert(fld, prec, StrLen(prec));
      FldGetAttributes(fld, &attr);
      attr.editable=false;
      attr.underlined=false;
      FldSetAttributes(fld, &attr);
      MemHandleUnlock(hrec);
      }*/
}

static void MindrefFormCleanup(Boolean ok)
{
   EventType e;
   
   if (ok) {
      VoidHand hrec;
      Ptr prec;
      Word selection;
      SysDBListItemType* pDBInfo;
      Byte type=PalmDBMindref;
      
      selection=LstGetSelection(GetObjectPtr(MindrefMindList));
      pDBInfo=MemHandleLock((VoidHand)hMindrefFormDBInfo);
      
      if (MindrefIndex==-1) {
	 /* create new ref */
	 CharPtr name;

	 name=FldGetTextPtr(GetObjectPtr(MindrefRefField));
	 if (!name)
	    name=pDBInfo[selection].name;
	 MindrefIndex=dmMaxRecordIndex;
	 /*FIXME hrec=DmNewRecord(MindrefDB, &MindrefIndex, StrLen(name)+2);
	 prec=MemHandleLock(hrec);
	 DmWrite(prec, 0, name, StrLen(name));
	 SetWord(prec, StrLen(name), 0);
	 MemHandleUnlock(hrec);
	 DmReleaseRecord(MindrefDB, MindrefIndex, true);*/
      }
      /*DBAppend(MindrefDB, MindrefIndex, &type, 1);
      DBAppend(MindrefDB, MindrefIndex, pDBInfo[selection].name,
      StrLen(pDBInfo[selection].name)+1);*/
      MemHandleUnlock((VoidHand)hMindrefFormDBInfo);
   } else {
      MindrefIndex=-1;
   }
   
   if (hMindrefFormDBInfo) {
      MemHandleUnlock((VoidHand)hMindrefFormDBInfo);
      MemHandleFree((VoidHand)hMindrefFormDBInfo);
   };

   e.eType=mindrefDoneEvent;
   EvtAddEventToQueue(&e);
}

Boolean MindrefFormHandleEvent(EventPtr e)
{
   Boolean handled=false;
   FormPtr frm;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      MindrefFormInit();
      frm=FrmGetActiveForm();
      FrmDrawForm(frm);
      FrmSetFocus(frm, FrmGetObjectIndex(frm, MindrefRefField));
      handled=true;
      break;

   case frmCloseEvent:
      MindrefFormCleanup(false);
      break;
      
   case ctlSelectEvent:
      switch (e->data.ctlSelect.controlID) {
      case DoneButton:
	 MindrefFormCleanup(true);
	 FrmReturnToForm(0);
	 handled=true;
	 break;

      case CancelButton:
	 MindrefFormCleanup(false);
	 FrmReturnToForm(0);
	 handled=true;
	 break;
      };
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
