/* newform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

typedef enum {NewLink, NewData, NewField} NewModeType;
static NewModeType NewMode;

static void NewFormInit()
{
   CtlSetValue(GetObjectPtr(NewLinkPushButton), 1);
   CtlSetValue(GetObjectPtr(NewChildPushButton), 1);
   NewMode=NewLink;
}

static void NewFormDraw()
{
   switch (NewMode) {
   case NewLink:
      WinInvertLine(0, 60, 160, 60);
      break;

   case NewData:
      WinInvertLine(0, 70, 160, 70);
      WinInvertLine(0, 110, 160, 110);
      break;

   case NewField:
      break;
   }
}

static void NewSetMode(NewModeType m)
{
   FormPtr frm=FrmGetActiveForm();
   
   if (m==NewMode) return;

   /* hide current page */
   NewFormDraw();
   switch (NewMode) {
   case NewLink:
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewLinkMasterLabel));
      CtlHideControl(GetObjectPtr(NewLinkMasterButton));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewTSLinkLabel));
      CtlHideControl(GetObjectPtr(NewBothWaysCheckBox));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewLinkDirLabel));
      CtlHideControl(GetObjectPtr(NewParentPushButton));
      CtlHideControl(GetObjectPtr(NewChildPushButton));
      CtlHideControl(GetObjectPtr(NewPeerPushButton));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewLinkLabelLabel));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewLabelField));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewLinkTargetLabel));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewTargetField));
      CtlHideControl(GetObjectPtr(NewBrowseButton));
      CtlHideControl(GetObjectPtr(NewAllowDupesCheckBox));
      CtlHideControl(GetObjectPtr(NewLinkButton));
      break;

   case NewData:
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewNoteLabel));
      CtlHideControl(GetObjectPtr(NewNoteButton));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewURLLabel));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewURLField));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewURLTitleLabel));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewURLTitleField));
      CtlHideControl(GetObjectPtr(NewURLButton));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewSearchLabel));
      CtlHideControl(GetObjectPtr(NewSearchTrigger));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewSearchForLabel));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewSearchField));
      CtlHideControl(GetObjectPtr(NewSearchButton));
      break;

   case NewField:
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewFieldTypeLabel));
      CtlHideControl(GetObjectPtr(NewFieldTextPushButton));
      CtlHideControl(GetObjectPtr(NewFieldCheckBoxPushButton));
      CtlHideControl(GetObjectPtr(NewFieldTimePushButton));
      CtlHideControl(GetObjectPtr(NewFieldMultiTextPushButton));
      CtlHideControl(GetObjectPtr(NewFieldNumberPushButton));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewFieldNameLabel));
      FrmHideObject(frm, FrmGetObjectIndex(frm, NewFieldNameField));
      CtlHideControl(GetObjectPtr(NewFieldButton));
      break;
   };

   /* show new page */
   NewMode=m;
   NewFormDraw();
   switch (NewMode) {
   case NewLink:
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewLinkMasterLabel));
      CtlShowControl(GetObjectPtr(NewLinkMasterButton));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewTSLinkLabel));
      CtlShowControl(GetObjectPtr(NewBothWaysCheckBox));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewLinkDirLabel));
      CtlShowControl(GetObjectPtr(NewParentPushButton));
      CtlShowControl(GetObjectPtr(NewChildPushButton));
      CtlShowControl(GetObjectPtr(NewPeerPushButton));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewLinkLabelLabel));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewLabelField));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewLinkTargetLabel));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewTargetField));
      CtlShowControl(GetObjectPtr(NewBrowseButton));
      CtlShowControl(GetObjectPtr(NewAllowDupesCheckBox));
      CtlShowControl(GetObjectPtr(NewLinkButton));
      break;

   case NewData:
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewNoteLabel));
      CtlShowControl(GetObjectPtr(NewNoteButton));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewURLLabel));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewURLField));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewURLTitleLabel));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewURLTitleField));
      CtlShowControl(GetObjectPtr(NewURLButton));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewSearchLabel));
      CtlShowControl(GetObjectPtr(NewSearchTrigger));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewSearchForLabel));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewSearchField));
      CtlShowControl(GetObjectPtr(NewSearchButton));
      break;

   case NewField:
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewFieldTypeLabel));
      CtlShowControl(GetObjectPtr(NewFieldTextPushButton));
      CtlShowControl(GetObjectPtr(NewFieldCheckBoxPushButton));
      CtlShowControl(GetObjectPtr(NewFieldTimePushButton));
      CtlShowControl(GetObjectPtr(NewFieldMultiTextPushButton));
      CtlShowControl(GetObjectPtr(NewFieldNumberPushButton));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewFieldNameLabel));
      FrmShowObject(frm, FrmGetObjectIndex(frm, NewFieldNameField));
      CtlShowControl(GetObjectPtr(NewFieldButton));
      break;
   };

}

static void NewNote()
{
   Char data[ItemHeaderSize+1]={0,ItemHeaderSize+1,NoteType, 0};
   DBAppend(IdeaDB, CurrentIndex, data, ItemHeaderSize+1);
}

static Boolean NewCtlSelect(EventPtr e)
{
   Boolean handled=false;
   
   switch (e->data.ctlSelect.controlID) {
   case Cancel:
      FrmReturnToForm(0);
      handled=true;
      break;
      
   case NewLinkPushButton:
      NewSetMode(NewLink);
      break;

   case NewDataPushButton:
      NewSetMode(NewData);
      break;

   case NewFieldPushButton:
      NewSetMode(NewField);
      break;

   case NewNoteButton:
      NewNote();
      FrmReturnToForm(0);
      FrmGotoForm(MainForm); /* force reopening of the form */
      handled=true;
      break;
   };
   return handled;
}

Boolean NewFormHandleEvent(EventPtr e)
{
   Boolean handled=false;
   FormPtr frm;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      NewFormInit();
      frm=FrmGetActiveForm();
      FrmDrawForm(frm);
      NewFormDraw();
      handled=true;
      break;

   case ctlSelectEvent:
      handled=NewCtlSelect(e);
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
