/* noteform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static VoidHand NoteHrec;

static void NoteFormInit()
{
   Ptr prec;
   Word offset, len;

   NoteHrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(NoteHrec);
   offset=ItemOffset(prec, ActiveItemIndex)+NoteTextOffset;
   len=StrLen(prec+offset)+1;
   MemHandleUnlock(NoteHrec);

   FldSetText(GetObjectPtr(NoteFormField), NoteHrec, offset, len);
}

static void NoteFormCleanup()
{
   FieldPtr fld=GetObjectPtr(NoteFormField);
   Boolean dirty=FldDirty(fld);
   Ptr prec;
   Word size;
   
   FldCompactText(fld);
   size=ItemHeaderSize+FldGetTextLength(fld)+1;
   FldSetTextHandle(fld, NULL);

   /* update size */
   prec=MemHandleLock(NoteHrec);
   SetWord(prec, ItemOffset(prec, ActiveItemIndex)+ItemSizeOffset, size);
   IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);
   MemHandleUnlock(NoteHrec);
   DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, dirty);

   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

Boolean NoteFormHandleEvent(EventPtr e)
{
   Boolean handled=false;
   FormPtr frm;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      NoteFormInit();
      frm=FrmGetActiveForm();
      FrmDrawForm(frm);
      FrmSetFocus(frm, FrmGetObjectIndex(frm, NoteFormField));
      handled=true;
      break;

   case frmCloseEvent:
      NoteFormCleanup();
      break;

   case ctlSelectEvent:
      switch (e->data.ctlSelect.controlID) {
      case DoneButton:
	 NoteFormCleanup();
	 FrmReturnToForm(0);
	 handled=true;
      };
      break;

   case menuEvent:
      handled=EditMenuHandleEvent(e->data.menu.itemID);
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
