/* noteform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static Handle SearchHAppList=NULL;
static SysDBListItemType* SearchPAppList;

static void SearchAppDrawFunc(UInt item, RectanglePtr bounds,
			      CharPtr* data)
{
   CharPtr s;
   CALLBACK_PROLOGUE;

   s=SearchPAppList[item].name;
   WinDrawChars(s, StrLen(s), bounds->topLeft.x, bounds->topLeft.y);

   CALLBACK_EPILOGUE;
};

static void SearchFormInit()
{
   ListPtr list;
   Word appcount;
   VoidHand hrec;
   Ptr prec;
   Word offset;
   ULong creator;
   int i;
   FieldPtr field;
   Word textlen;

   list=GetObjectPtr(SearchList);
   if (!SearchHAppList) {
      if (!SysCreateDataBaseList('appl', 0, &appcount, &SearchHAppList, true))
	 FrmAlert(AppListErrorAlert);
      else {
	 LstSetListChoices(list, NULL, appcount);
	 LstSetDrawFunction(list, SearchAppDrawFunc);
	 SearchPAppList=MemHandleLock((VoidHand)SearchHAppList);
      };
   };

   /* open record */
   hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
      
   /* set selection */
   prec=MemHandleLock(hrec);
   offset=ItemOffset(prec, ActiveItemIndex);
   creator=GetLong(prec, offset+SearchCreatorOffset);
   textlen=StrLen(prec+offset+SearchTextOffset);
   MemHandleUnlock(hrec);
   if (creator)
      for (i=0; i<appcount; i++) 
	 if (SearchPAppList[i].creator==creator) {
	    LstSetSelection(list, i);
	    CtlSetLabel(GetObjectPtr(SearchTrigger), SearchPAppList[i].name);
	 };

   /* set field */
   field=GetObjectPtr(SearchField);
   FldSetText(field, hrec, offset+SearchTextOffset, textlen+1);
   FldSetDirty(field, false);
}

static void SearchSetApp(Word index)
{
   VoidHand hrec;
   Ptr prec;
   
   CtlSetLabel(GetObjectPtr(SearchTrigger), SearchPAppList[index].name);

   /* write to record */
   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   SetLong(prec, ItemOffset(prec, ActiveItemIndex)+SearchCreatorOffset,
	   SearchPAppList[index].creator);
   MemHandleUnlock(hrec);
   FldSetDirty(GetObjectPtr(SearchField), true);
}

static void SearchFormCleanup()
{
   FieldPtr field;
   VoidHand hrec;
   Ptr prec;
   
   MemHandleUnlock((VoidHand)SearchHAppList);
   MemHandleFree((VoidHand)SearchHAppList);
   SearchHAppList=NULL;

   field=GetObjectPtr(SearchField);
   FldCompactText(field);
   /* update length */
   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   SetWord(prec, ItemOffset(prec, ActiveItemIndex)+ItemSizeOffset,
	   SearchTextOffset+FldGetTextLength(field)+1);
   FldSetTextHandle(field, NULL);
   if (FldDirty(field)) {
      IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);
      DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, true);
   } else
      DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, false);

   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

Boolean SearchFormHandleEvent(EventPtr e)
{
   Boolean handled=false;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      SearchFormInit();
      FrmDrawForm(FrmGetActiveForm());
      handled=true;
      break;

   case frmCloseEvent:
      SearchFormCleanup();
      break;

   case ctlSelectEvent:
      switch (e->data.ctlSelect.controlID) {
      case DoneButton:
	 SearchFormCleanup();
	 FrmReturnToForm(0);
	 handled=true;
      };
      break;

   case popSelectEvent:
      switch (e->data.popSelect.listID) {
      case SearchList:
	 SearchSetApp(e->data.popSelect.selection);
	 handled=true;
	 break;
      };
      break;

   case menuEvent:
      handled=EditMenuHandleEvent(e->data.menu.itemID);
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
