/* thoughtstream.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

/* Globals */
DmOpenRef IdeaDB;
/*DmOpenRef SyncDB;
  DmOpenRef MindrefDB;*/

ULong CurrentID;
Word CurrentIndex;

Word ActiveItemIdeaIndex;
SWord ActiveItemIndex;

/* The standard random function is only 16 bits, but we need 32 bits */
ULong Random32()
{
   union {
      ULong ulong;
      struct {
	 Word word1,word2;
      } s;
   } u;

   u.s.word1=SysRandom(0);
   u.s.word2=SysRandom(0);
   return u.ulong;
}

/* this function from oreilly's palm programming book */
Boolean EditMenuHandleEvent(Word menuid)
{
   FieldPtr fld;

   switch (menuid) {
   case EditUndo:
   case EditCut:
   case EditCopy:
   case EditPaste:
   case EditSelectAll:
      fld=GetFocusObjectPtr();
      if (!fld)
	 return false;
      if (menuid==EditUndo)
	 FldUndo(fld);
      else if (menuid==EditCut)
	 FldCut(fld);
      else if (menuid==EditCopy)
	 FldCopy(fld);
      else if (menuid==EditPaste)
	 FldPaste(fld);
      else if (menuid==EditSelectAll)
	 FldSetSelection(fld, 0, FldGetTextLength(fld));
      return true;

   case EditKeyboard:
      SysKeyboardDialog(kbdDefault);
      return true;

   case EditGraffiti:
      SysGraffitiReferenceDialog(referenceDefault);
      return true;

   default:
      return false;
   };
}

static void TSFind(FindParamsPtr findparams)
{
   Boolean done=false;

   findparams->more=false;
   done=FindDrawHeader(findparams, "ThoughtStream");
   if (done)
      findparams->more=true;
   else {
      Word index=findparams->recordNum;
      DmOpenRef db;
      DmSearchStateType dmsearchstate;
      UInt cardno;
      LocalID dbid;

      DmGetNextDatabaseByTypeCreator(true, &dmsearchstate,
				     'MIND', 'ThSt', true, &cardno, &dbid);
      db=DmOpenDatabase(cardno, dbid, findparams->dbAccesMode);
      if (!db)
	 return;

      while (1) {
	 VoidHand hrec;
	 Ptr prec;
	 Boolean match=false;
	 Word i, pos, fieldnum=0;

	 hrec=DmQueryRecord(db, index);
	 if (!hrec)
	    break;
	 prec=MemHandleLock(hrec);

	 if (FindStrInStr(prec+IdeaNameOffset,
			  findparams->strToFind, &pos)) {
	    match=true;
	    fieldnum=0;
	 } else for (i=0; i<IdeaNumItems(prec); i++) {
	    Word itemoffset=ItemOffset(prec, i);
	    SWord dataoffset1=-1, dataoffset2=-1;

	    switch (GetByte(prec, itemoffset+ItemTypeOffset)) {
	    case NoteType:
	       dataoffset1=NoteTextOffset;
	       break;

	    case ParentType:
	    case ChildType:
	    case PeerType:
	       dataoffset1=LinkLabelOffset;
	       dataoffset2=LinkTargetTitleOffset(prec+itemoffset);
	       break;

	    case URLType:
	       dataoffset1=URLTitleOffset;
	       dataoffset2=URLAddressOffset(prec+itemoffset);
	       break;

	    case SearchType:
	       dataoffset1=SearchTextOffset;
	       break;

	    case LMType:
	       dataoffset1=LMTextOffset;
	       break;

	    default:
	       break;
	    };
	    if (dataoffset1!=-1) {
	       if (FindStrInStr(prec+itemoffset+dataoffset1,
				findparams->strToFind, &pos)) {
		  match=true;
		  fieldnum=i;
		  break;
	       };
	    } else if (dataoffset2!=-1) {
	       if (FindStrInStr(prec+itemoffset+dataoffset2,
				findparams->strToFind, &pos)) {
		  match=true;
		  fieldnum=i;
		  break;
	       };
	    };
	 };
	 if (match) {
	    RectangleType rect;

	    done=FindSaveMatch(findparams, index, pos, fieldnum,
			       GetLong(prec, IdeaIDOffset), cardno, dbid);
	    if (done) {
	       MemHandleUnlock(hrec);
	       break;
	    };

	    FindGetLineBounds(findparams, &rect);
	    WinDrawChars(prec+IdeaNameOffset, StrLen(prec+IdeaNameOffset),
			 rect.topLeft.x, rect.topLeft.y);
	    findparams->lineNumber++;
	 };
	 MemHandleUnlock(hrec);
	 if (done)
	    break;
	 index++;
      };
      DmCloseDatabase(db);
   };       
}

static void SendGotoEvent(GoToParamsPtr gotoparams, TSLinkPtr linkinfo)
{
   EventType event;

   event.eType=frmGotoEvent;
   event.data.frmGoto.formID=MainForm;
   if (gotoparams) {
      event.data.frmGoto.matchCustom=gotoparams->matchCustom;
      event.data.frmGoto.matchFieldNum=gotoparams->matchFieldNum;
      
      /* kludge alert - pack cardno/dbid into recordnum,
	 matchpos, and matchlen */
      event.data.frmGoto.recordNum=gotoparams->dbCardNo;
      event.data.frmGoto.matchPos=gotoparams->dbID&0xffff;
      event.data.frmGoto.matchLen=gotoparams->dbID>>16;
   } else if (linkinfo) {
      LocalID dbid;
      
      event.data.frmGoto.matchCustom=linkinfo->id;
      event.data.frmGoto.matchFieldNum=0;

      dbid=DmFindDatabase(0, linkinfo->db_name);
      /* repeated kludge */
      event.data.frmGoto.recordNum=0;
      event.data.frmGoto.matchPos=dbid&0xffff;
      event.data.frmGoto.matchLen=dbid>>16;
   };      

   EvtAddEventToQueue(&event);
}

TSLinkPtr TSLinkNew(DmOpenRef db, Word index)
{
   TSLinkPtr linkinfo;
   VoidHand hrec;
   Ptr prec;
   CharPtr title;
   int titlelen;
   LocalID dbid;
   UInt cardno;

   hrec=DmQueryRecord(db, index);
   prec=MemHandleLock(hrec);
   title=prec+IdeaNameOffset;
   titlelen=StrLen(title);
   linkinfo=MemPtrNew(64+titlelen+1);
   MemSet(linkinfo, 64+titlelen+1, 0);
   ErrFatalDisplayIf(!linkinfo, "Not enough memory");
   StrNCopy(((CharPtr)linkinfo)+64, title, titlelen);
   ((CharPtr)linkinfo)[64+titlelen]=0;
   MemHandleUnlock(hrec);

   linkinfo->creator='ThSt';
   linkinfo->id=GetLong(prec,IdeaIDOffset);

   DmOpenDatabaseInfo(db, &dbid, NULL, NULL, &cardno, NULL);
   DmDatabaseInfo(cardno, dbid, linkinfo->db_name,
		  NULL, NULL, NULL, NULL, NULL,
		  NULL, NULL, NULL, NULL, NULL);

   return linkinfo;
}


static Boolean ApplicationHandleEvent(EventPtr e)
{
   FormPtr frm;
   Word    formId;
   Boolean handled = false;

   if (e->eType == frmLoadEvent) {
      formId = e->data.frmLoad.formID;
      frm = FrmInitForm(formId);
      FrmSetActiveForm(frm);

      switch(formId) {
      case MainForm:
	 FrmSetEventHandler(frm, MainFormHandleEvent);
	 break;

      case MindForm:
	 FrmSetEventHandler(frm, MindFormHandleEvent);
	 break;

      case NoteForm:
	 FrmSetEventHandler(frm, NoteFormHandleEvent);
	 break;

      case LinkForm:
	 FrmSetEventHandler(frm, LinkFormHandleEvent);
	 break;

      case BrowseForm:
	 FrmSetEventHandler(frm, BrowseFormHandleEvent);
	 break;

      case URLForm:
	 FrmSetEventHandler(frm, URLFormHandleEvent);
	 break;

      case SearchForm:
	 FrmSetEventHandler(frm, SearchFormHandleEvent);
	 break;

      case PrefsForm:
	 FrmSetEventHandler(frm, PrefsFormHandleEvent);
	 break;

      case LMForm:
	 FrmSetEventHandler(frm, LMFormHandleEvent);
	 break;

      case FieldForm:
	 FrmSetEventHandler(frm, FieldFormHandleEvent);
	 break;

      case MindrefForm:
	 FrmSetEventHandler(frm, MindrefFormHandleEvent);
	 break;
      }
      handled = true;
   }

   return handled;
}

/* Get preferences, open (or create) app database */
static Word StartApplication(TSLinkPtr linkinfo, GoToParamsPtr gotoparams)
{
   ReadPrefs();

   if (linkinfo) {
      if (MindOpen(linkinfo->db_name)) {
	 CurrentID=linkinfo->id;
	 FrmGotoForm(MainForm);
	 return 0;
      };
   } else if (gotoparams) {
      Char dbname[33];
      
      DmDatabaseInfo(gotoparams->dbCardNo, gotoparams->dbID, dbname, NULL,
		     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
      if (MindOpen(dbname)) {
	 CurrentID=gotoparams->matchCustom;
	 FrmGotoForm(MainForm);
	 return 0;
      };
   }
   
   /* attempt to restore previous idea */
   if (prefs.lastID) {
      if (MindOpen(prefs.last_mind)) {
	 CurrentID=prefs.lastID;
	 FrmGotoForm(MainForm);
	 return 0;
      };
   };
   
   /* can't restore state, so let user open or create a mind */
   FrmGotoForm(MindForm);
   return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication()
{
   WritePrefs();
   FrmSaveAllForms();
   FrmCloseAllForms();
}

/* The main event loop
   The return value is used for closing dialogs */
Word EventLoop()
{
   Word err;
   EventType e;

   do {
      EvtGetEvent(&e, evtWaitForever);
      if (! SysHandleEvent (&e))
	 if (! MenuHandleEvent (NULL, &e, &err))
	    if (! ApplicationHandleEvent (&e))
	       FrmDispatchEvent (&e);
   } while ((e.eType!=appStopEvent)&&(e.eType!=dlgCloseEvent));
   if (e.eType==dlgCloseEvent)
      return e.data.generic.data1;
   else
      return 0;
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
   Word err;

   switch (cmd) {
   case sysAppLaunchCmdNormalLaunch:
      err = StartApplication(NULL, NULL);
      if (err) return err;

      EventLoop();
      StopApplication();
      break;

   case linkAppLaunchRequestLink:
      err=StartApplication(NULL, NULL);
      if (err) return err;
      EventLoop();
      StopApplication();
      break;

   case sysAppLaunchCmdSaveData:
      FrmSaveAllForms();
      break;

   case sysAppLaunchCmdFind:
      TSFind((FindParamsPtr)cmdPBP);
      break;

   case sysAppLaunchCmdGoTo:
      if (launchFlags&sysAppLaunchFlagNewGlobals) {
	 err=StartApplication(NULL, (GoToParamsPtr)cmdPBP);
	 if (err) return err;
	 EventLoop();
	 StopApplication();
      } else
	 SendGotoEvent((GoToParamsPtr)cmdPBP, NULL);
      break;

   case sysAppLaunchCmdSyncNotify:
      /* FIXME: sort stuff */
      break;

   case linkAppLaunchFollowLink:
      err=StartApplication((TSLinkPtr)cmdPBP, NULL);
      if (err) return err;
      EventLoop();
      StopApplication();
      break;
      
   case linkAppLaunchFollowLinkSubCall:
      if (launchFlags&sysAppLaunchFlagSubCall) 
	 SendGotoEvent(NULL, (TSLinkPtr)cmdPBP);
      else {
	 UInt cardno;
	 LocalID dbid;
	 
	 SysCurAppDatabase(&cardno, &dbid);
	 SysUIAppSwitch(cardno, dbid, linkAppLaunchFollowLink, cmdPBP);
      }
      break;

   case linkAppLaunchReturnLink:
      LMStoreLink((TSLinkPtr)cmdPBP);
      err=StartApplication((TSLinkPtr)cmdPBP, NULL);
      if (err) return err;
      EventLoop();
      StopApplication();
      break;
            
   default:
      return sysErrParamErr;
   };

   return 0;
}
