/* urlform.c
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>
#include "callback.h"

#include "thoughtstream.h"

static void URLFormInit()
{
   FieldPtr titlefield, addressfield;
   VoidHand hrec;
   Ptr prec;
   Word itemoffset;

   hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
   prec=MemHandleLock(hrec);
   itemoffset=ItemOffset(prec, ActiveItemIndex);

   titlefield=GetObjectPtr(URLTitleField);
   FldInsert(titlefield, prec+itemoffset+URLTitleOffset,
	     StrLen(prec+itemoffset+URLTitleOffset));
   FldSetDirty(titlefield, false);

   addressfield=GetObjectPtr(URLAddressField);
   FldInsert(addressfield, prec+itemoffset+URLAddressOffset(prec+itemoffset),
	     StrLen(prec+itemoffset+URLAddressOffset(prec+itemoffset)));
   FldSetDirty(addressfield, false);

   MemHandleUnlock(hrec);
}

static void URLFormCleanup()
{
   FieldPtr field;
   VoidHand hrec;
   Ptr prec;
   Word itemoffset, oldlen, newlen;

   field=GetObjectPtr(URLTitleField);
   if (FldDirty(field)) {
      hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
      prec=MemHandleLock(hrec);
      itemoffset=ItemOffset(prec, ActiveItemIndex);
      oldlen=StrLen(prec+itemoffset+URLTitleOffset);
      MemHandleUnlock(hrec);
      DBRemove(IdeaDB, ActiveItemIdeaIndex, itemoffset+URLTitleOffset, oldlen);
      newlen=FldGetTextLength(field);
      if (newlen)
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, itemoffset+URLTitleOffset,
		  FldGetTextPtr(field), newlen);
      hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
      prec=MemHandleLock(hrec);
      SetWord(prec, itemoffset+ItemSizeOffset,
	      GetWord(prec, itemoffset+ItemSizeOffset)-oldlen+newlen);
      IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);
      MemHandleUnlock(hrec);
      DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, true);
   }

   field=GetObjectPtr(URLAddressField);
   if (FldDirty(field)) {
      Word stringoffset;
      
      hrec=DmQueryRecord(IdeaDB, ActiveItemIdeaIndex);
      prec=MemHandleLock(hrec);
      itemoffset=ItemOffset(prec, ActiveItemIndex);
      stringoffset=itemoffset+URLAddressOffset(prec+itemoffset);
      oldlen=StrLen(prec+stringoffset);
      MemHandleUnlock(hrec);
      DBRemove(IdeaDB, ActiveItemIdeaIndex, stringoffset, oldlen);
      newlen=FldGetTextLength(field);
      if (newlen)
	 DBInsert(IdeaDB, ActiveItemIdeaIndex, stringoffset,
		  FldGetTextPtr(field), newlen);
      hrec=DmGetRecord(IdeaDB, ActiveItemIdeaIndex);
      prec=MemHandleLock(hrec);
      SetWord(prec, itemoffset+ItemSizeOffset,
	      GetWord(prec, itemoffset+ItemSizeOffset)-oldlen+newlen);
      IdeaSetDirty(IdeaDB, ActiveItemIdeaIndex);
      MemHandleUnlock(hrec);
      DmReleaseRecord(IdeaDB, ActiveItemIdeaIndex, true);
   }

   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

Boolean URLFormHandleEvent(EventPtr e)
{
   Boolean handled=false;

   CALLBACK_PROLOGUE;

   switch (e->eType) {
   case frmOpenEvent:
      URLFormInit();
      FrmDrawForm(FrmGetActiveForm());
      handled=true;
      break;

   case frmCloseEvent:
      URLFormCleanup();
      break;

   case ctlSelectEvent:
      switch (e->data.ctlSelect.controlID) {
      case DoneButton:
	 URLFormCleanup();
	 FrmReturnToForm(0);
	 handled=true;
      };
      break;

   case menuEvent:
      handled=EditMenuHandleEvent(e->data.menu.itemID);
      break;

   default:
      break;
   };
   CALLBACK_EPILOGUE;
   return handled;
}
