/* wrappers.c - wrappers for common PalmOS tasks
   Copyright (C) 1999 Ben Darnell
   This file is a part of ThoughtStream,
   which is distributed under the GNU GPL.
*/

#include <Pilot.h>

#include "wrappers.h"

VoidPtr GetObjectPtr(Word id)
{
   FormPtr frm;
   frm=FrmGetActiveForm();
   return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, id));
}

DmOpenRef DBCreate(ULong creator, ULong type, CharPtr name)
{
   DmOpenRef db;
   Err err;
   LocalID dbid;
   UInt attr;

   err=DmCreateDatabase(0, name, creator, type, false);
   if (err) return 0;

   dbid=DmFindDatabase(0, name);
   db=DmOpenDatabase(0, dbid, dmModeReadWrite);

   DmDatabaseInfo(0, dbid, NULL, &attr, NULL,
		  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
   attr|=dmHdrAttrBackup;
   DmSetDatabaseInfo(0, dbid, NULL, &attr, NULL,
		     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

   return db;
}

Byte GetByte(Ptr prec, Word offset)
{
   return prec[offset];
}

Word GetWord(Ptr prec, Word offset)
{
   union {
      Word n;
      char c[2];
   } u;

   u.c[0]=prec[offset];
   u.c[1]=prec[offset+1];

   return u.n;
}

Long GetLong(Ptr prec, Word offset)
{
   union {
      Long n;
      char c[4];
   } u;

   u.c[0]=prec[offset];
   u.c[1]=prec[offset+1];
   u.c[2]=prec[offset+2];
   u.c[3]=prec[offset+3];

   return u.n;
}

void SetByte(Ptr prec, Word offset, Byte data)
{
   DmWrite(prec, offset, &data, 1);
}

void SetWord(Ptr prec, Word offset, Word data)
{
   DmWrite(prec, offset, &data, 2);
}

void SetLong(Ptr prec, Word offset, Long data)
{
   DmWrite(prec, offset, &data, 4);
}

Err DBInsert(DmOpenRef db, Word record_index, Word offset,
	     Ptr src, Word num_bytes)
{
   VoidHand hrec;
   Ptr prec;
   Err err;
   Word oldsize;

   hrec=DmGetRecord(db, record_index);
   oldsize=MemHandleSize(hrec);
   hrec=DmResizeRecord(db, record_index,
		       oldsize+num_bytes);
   prec=MemHandleLock(hrec);
   err=DmWrite(prec, offset+num_bytes, prec+offset, oldsize-offset);
   if ((!err)&&(src))
      err=DmWrite(prec, offset, src, num_bytes);
   MemHandleUnlock(hrec);
   DmReleaseRecord(db, record_index, true);
   return err;
}

Err DBAppend(DmOpenRef db, Word record_index, Ptr src, Word num_bytes)
{
   VoidHand hrec;
   Ptr prec;
   Err err=0;
   Word oldsize;

   hrec=DmGetRecord(db, record_index);
   oldsize=MemHandleSize(hrec);
   hrec=DmResizeRecord(db, record_index,
		       oldsize+num_bytes);
   prec=MemHandleLock(hrec);
   if (src)
      err=DmWrite(prec, oldsize, src, num_bytes);
   DmReleaseRecord(db, record_index, true);
   MemHandleUnlock(hrec);
   return err;
}

Err DBRemove(DmOpenRef db, Word record_index, Word offset, Word num_bytes)
{
   VoidHand hrec;
   Ptr prec;
   Err err;
   Word oldsize;

   hrec=DmGetRecord(db, record_index);
   oldsize=MemHandleSize(hrec);
   prec=MemHandleLock(hrec);
   err=DmWrite(prec, offset, prec+offset+num_bytes, oldsize-offset-num_bytes);
   MemHandleUnlock(hrec);
   hrec=DmResizeRecord(db, record_index, oldsize-num_bytes);
   DmReleaseRecord(db, record_index, true);
   return err;
}

/* this function from oreilly's palm programming book */
FieldPtr GetFocusObjectPtr()
{
   FormPtr frm;
   Word focus;
   FormObjectKind objType;

   frm=FrmGetActiveForm();
   focus=FrmGetFocus(frm);
   if (focus==noFocus)
      return NULL;

   objType=FrmGetObjectType(frm, focus);

   if (objType==frmFieldObj)
      return FrmGetObjectPtr(frm, focus);
   else if (objType==frmTableObj)
      return TblGetCurrentField(FrmGetObjectPtr(frm, focus));

   return NULL;
}

void Beep(Long freq, UInt duration)
{
   SndCommandType c;
   c.cmd=sndCmdFreqDurationAmp;
   c.param1=freq;
   c.param2=duration;
   c.param3=sndMaxAmp-1;
   SndDoCmd(NULL, &c, false);
}
